#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.container.ListOf import ListOf

from libsignetsim.sedml.ChangeAttribute import ChangeAttribute
from libsignetsim.sedml.ComputeChange import ComputeChange
from libsignetsim.sedml.AddXML import AddXML
from libsignetsim.settings.Settings import Settings

import libsbml
from libsedml import SEDML_CHANGE_ATTRIBUTE, SEDML_CHANGE_COMPUTECHANGE, SEDML_CHANGE_ADDXML
from six.moves import reload_module
reload_module(libsbml)

class ListOfChanges(ListOf):

	def __init__(self, document, model):

		ListOf.__init__(self, document)

		self.__document = document
		self.__model = model
		self.__changeCounter = 0

	def new(self, change_type, change_id=None):
		t_change_id = change_id
		if t_change_id is None:
			t_change_id = "%s_change_%d" % (self.__model.getId(), self.__changeCounter)

		if change_type == SEDML_CHANGE_ATTRIBUTE:
			t_change = ChangeAttribute(self.__document)
			t_change.setId(t_change_id)
			ListOf.append(self, t_change)
			self.__changeCounter += 1
			return t_change

		elif change_type == SEDML_CHANGE_COMPUTECHANGE:
			t_change = ComputeChange(self.__document)
			t_change.setId(t_change_id)
			ListOf.append(self, t_change)
			self.__changeCounter += 1
			return t_change

		elif change_type == SEDML_CHANGE_ADDXML:
			t_change = AddXML(self.__document)
			t_change.setId(t_change_id)
			ListOf.append(self, t_change)
			self.__changeCounter += 1
			return t_change

	def createChangeAttribute(self, change_id=None):
		return self.new(SEDML_CHANGE_ATTRIBUTE, change_id)

	def createComputeChange(self, change_id=None):
		return self.new(SEDML_CHANGE_COMPUTECHANGE, change_id)

	def createAddXML(self, change_id=None):
		return self.new(SEDML_CHANGE_ADDXML, change_id)

	def readSedml(self, list_of_changes, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.readSedml(self, list_of_changes, level, version)

		for change in list_of_changes:

			if change.getTypeCode() == SEDML_CHANGE_ATTRIBUTE:
				t_change = ChangeAttribute(self.__document)
				t_change.readSedml(change, level, version)
				ListOf.append(self, t_change)
				self.__changeCounter += 1

			elif change.getTypeCode() == SEDML_CHANGE_COMPUTECHANGE:
				t_change = ComputeChange(self.__document)
				t_change.readSedml(change, level, version)
				ListOf.append(self, t_change)
				self.__changeCounter += 1

			elif change.getTypeCode() == SEDML_CHANGE_ADDXML:
				t_change = AddXML(self.__document)
				t_change.readSedml(change, level, version)
				ListOf.append(self, t_change)
				self.__changeCounter += 1

	def writeSedml(self, list_of_changes, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.writeSedml(self, list_of_changes, level, version)

		for change in self:
			if isinstance(change, ChangeAttribute):
				t_change = list_of_changes.createChangeAttribute()
				change.writeSedml(t_change, level, version)

			elif isinstance(change, ComputeChange):
				t_change = list_of_changes.createComputeChange()
				change.writeSedml(t_change, level, version)

			elif isinstance(change, AddXML):
				t_change = list_of_changes.createAddXML()
				change.writeSedml(t_change, level, version)

	def applyChanges(self, sbml_model):

		for change in self:
			if isinstance(change, ChangeAttribute) or isinstance(change, ComputeChange):
				change.applyChange(sbml_model)

	def applyXMLChanges(self, sbml_tree):
		for change in self:
			if isinstance(change, AddXML):
				change.applyChange(sbml_tree)

	def nbXMLChanges(self):

		counter = 0
		for change in self:
			if isinstance(change, AddXML):
				counter += 1
		return counter


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.container.ListOf import ListOf
from libsignetsim.model.sbml.container.HasIds import HasIds
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.sbml.HasParentObj import HasParentObj
from libsignetsim.model.sbml.Reaction import Reaction
from libsignetsim.model.ModelException import InvalidXPath
from libsignetsim.settings.Settings import Settings

from re import match


class ListOfReactions(ListOf, HasIds, SbmlObject, HasParentObj):
	""" Class for the list of reactions in a sbml model """

	def __init__ (self, model, parent_obj):

		self.__model = model
		ListOf.__init__(self, model)
		HasIds.__init__(self, model)
		SbmlObject.__init__(self, model)
		HasParentObj.__init__(self, parent_obj)

	def readSbml(self, sbml_listOfReactions,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Reads a list of reactions from a sbml file """

		for sbml_reaction in sbml_listOfReactions:
			t_reaction = Reaction(self.__model, self, self.nextId())
			ListOf.add(self, t_reaction)
			t_reaction.readSbml(sbml_reaction, sbml_level, sbml_version)

		SbmlObject.readSbml(self, sbml_listOfReactions, sbml_level, sbml_version)


	def writeSbml(self, sbml_model,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Writes a list of reactions from a sbml file """

		for reaction in self:
			reaction.writeSbml(sbml_model, sbml_level, sbml_version)

		if len(self):
			SbmlObject.writeSbml(self, sbml_model.getListOfReactions(), sbml_level, sbml_version)

	def new(self, name=None):

		t_reaction = Reaction(self.__model, self, self.nextId())
		t_reaction.new(name)
		ListOf.add(self, t_reaction)
		return t_reaction

	def copy(self, obj, deletions=[], sids_subs={}, symbols_subs={}, usids_subs={}, conversion_factors={},
				extent_conversion=None, time_conversion=None):

		if obj not in deletions:
			SbmlObject.copy(self, obj)
			for reaction in obj:
				if reaction not in deletions:

					t_reaction = Reaction(self.__model, self, self.nextId())
					t_reaction.copy(
						reaction,
						deletions=deletions,
						sids_subs=sids_subs,
						symbols_subs=symbols_subs,
						usids_subs=usids_subs,
						conversion_factors=conversion_factors,
						extent_conversion=extent_conversion,
						time_conversion=time_conversion
					)
					ListOf.add(self, t_reaction)

	def remove(self, reaction):

		self.__model.listOfVariables.removeVariable(reaction)
		ListOf.remove(self, reaction)

	def containsVariable(self, variable):
		for reaction in self:
			if reaction.containsVariable(variable):
				return True
		return False

	def containsLocalParameterSbmlId(self, sbml_id):

		for reaction in self:
			if reaction.listOfLocalParameters.containsSbmlId(sbml_id):
				return True
		return False


	def getLocalParameterReactionIdBySbmlId(self, sbml_id):

		for reaction in self:
			if reaction.listOfLocalParameters.containsSbmlId(sbml_id):
				return reaction.objId
		return -1


	def getLocalParameterIdBySbmlId(self, sbml_id):

		for reaction in self:
			if reaction.listOfLocalParameters.containsSbmlId(sbml_id):
				return reaction.listOfLocalParameters.getBySbmlId(sbml_id).objId
		return -1

	def countLocalParameters(self):

		nb_local_parameters = 0
		for reaction in self:
			nb_local_parameters += len(reaction.listOfLocalParameters)

		return nb_local_parameters

	def hasFastReaction(self):

		for reaction in self:
			if reaction.fast:
				return True
		return False

	def hasVariableStoichiometry(self):

		for reaction in self:
			if reaction.hasVariableStoichiometry():
				return True
		return False

	def renameSbmlId(self, old_sbml_id, new_sbml_id):

		for obj in self:
			obj.renameSbmlId(old_sbml_id, new_sbml_id)



	def resolveXPath(self, selector):

		if not (selector.startswith("reaction") or selector.startswith("sbml:reaction")):
			raise InvalidXPath(selector)

		res_match = match(r'(.*)\[@(.*)=(.*)\]', selector)
		if res_match is None:
			raise InvalidXPath(selector)

		tokens = res_match.groups()
		if len(tokens) != 3:
			raise InvalidXPath(selector)

		object = None
		if tokens[1] == "id":
			object = self.getBySbmlId(tokens[2][1:-1])
		elif tokens[1] == "name":
			object = self.getByName(tokens[2][1:-1])
		elif tokens[1] == "metaid":
			object = self.getByMetaId(tokens[2][1:-1])

		if object is not None:
			return object

		# If not returned yet
		raise InvalidXPath(selector)

	def getByXPath(self, xpath):
		return self.resolveXPath(xpath[0]).getByXPath(xpath[1:])

	def setByXPath(self, xpath, object):
		self.resolveXPath(xpath[0]).setByXPath(xpath[1:], object)

	def getXPath(self):
		return "/".join([self.getParentObj().getXPath(), "sbml:listOfReactions"])




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Testing the reading/writing of miriam annotations of the model

"""

from libsignetsim import SbmlDocument, Settings

from unittest import TestCase
from os.path import join, dirname
from os import getcwd


class TestMetaIds(TestCase):
	""" Tests high level functions """


	def testMetaIds(self):

		testfiles_path = join(join(getcwd(), dirname(__file__)), "files")

		sbml_doc = SbmlDocument()
		model = sbml_doc.getModelInstance()

		s1 = model.listOfSpecies.new("A")
		s2 = model.listOfSpecies.new("B")
		s3 = model.listOfSpecies.new("C")


		# Writing with defaults meta ids
		sbml_doc.writeSbmlToFile(join(Settings.tempDirectory, "test_meta_ids_1.xml"))

		# Forcing s2 to meta id '_meta_id_0_', which force the renaming of the existing one (the model)
		s2.setMetaId("_meta_id_0_", force=True)

		self.assertEqual(s2.getMetaId(), "_meta_id_0_")
		self.assertEqual(model.getMetaId(), "_meta_id_15_")

		# Now forcing the model to have no defined meta id.
		model.unsetMetaId()
		self.assertEqual(model.getMetaId(), None)

		sbml_doc.writeSbmlToFile(join(Settings.tempDirectory, "test_meta_ids_2.xml"))

		# During reading, compartment should get the default '_meta_id_0_' meta id. But when reading s2,
		# it should be renamed

		sbml_doc_2 = SbmlDocument()
		sbml_doc_2.readSbmlFromFile(join(Settings.tempDirectory, "test_meta_ids_2.xml"))
		model_2 = sbml_doc_2.getModelInstance()

		# for key, obj in model_2.listOfSbmlObjects.items():
		# 	print "%s:%s (%s)" % (key, obj.getMetaId(), type(obj))

		s2 = model_2.listOfSpecies.getByName("B")
		self.assertEqual(s2.getMetaId(), "_meta_id_0_")
		self.assertEqual(model_2.getMetaId(), "_meta_id_16_")

		sbml_doc_2.writeSbmlToFile(join(Settings.tempDirectory, "test_meta_ids_3.xml"))

		sbml_doc_3 = SbmlDocument()
		sbml_doc_3.readSbmlFromFile(join(Settings.tempDirectory, "test_meta_ids_3.xml"))
		model_3 = sbml_doc_3.getModelInstance()

		s2 = model_3.listOfSpecies.getByName("B")
		self.assertEqual(s2.getMetaId(), "_meta_id_0_")
		self.assertEqual(model_3.getMetaId(), "_meta_id_16_")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file is made for 'high level' tests, using various components

"""

from libsignetsim import SbmlDocument, Experiment, ModelVsTimeseriesOptimization

from unittest import TestCase
from os.path import join, dirname


class TestOptimizationComp(TestCase):
	""" Tests high level functions """

	def testOptimizeCompModel(self):

		doc = SbmlDocument()
		doc.readSbmlFromFile(join(dirname(__file__), "files", "comp_model", "modelz9xdww.xml"))

		experiment = Experiment()
		experiment.readNuMLFromFile(join(join(dirname(__file__), "files"), "ras_data.xml"))

		# Fitting the hierarchical model
		sos_submodel = doc.model.listOfSubmodels.getBySbmlId("sos_mod").getModelObject()
		param_def = sos_submodel.listOfParameters.getByName("SOS inactivation by Mapk catalytic constant")

		selected_parameters = [(param_def, 1, 1e-6, 1e+6, 8)]

		fit = ModelVsTimeseriesOptimization(
			workingModel=doc.model,
			list_of_experiments=[experiment],
			parameters_to_fit=selected_parameters,
			p_lambda=1, p_freeze_count=1
		)

		score_def = fit.runOptimization(2)
		res_def = fit.readOptimizationOutput()

		self.assertEqual(list(res_def.keys())[0], param_def)
		self.assertAlmostEqual(score_def, 1.006, delta=1e-3)
		self.assertAlmostEqual(res_def[param_def], 18.87788, delta=res_def[param_def]*1e-4)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import division

from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.settings.Settings import Settings
from libsignetsim.model.math.MathDevelopper import unevaluatedSubs
from libsignetsim.model.math.sympy_shortcuts import SympyFloat, SympyInteger


class EventDelay(SbmlObject, MathFormula):
	""" Events priority's definition """

	def __init__(self, model, math_only=False):

		self.__model = model
		MathFormula.__init__(self, model, MathFormula.MATH_DELAY)

		# For math submodels, where objects are not sbml objects
		self.mathOnly = math_only
		if not self.mathOnly:
			SbmlObject.__init__(self, model)

	def readSbml(self, sbml_delay,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Reads an event priority definition from a sbml file """

		SbmlObject.readSbml(self, sbml_delay, sbml_level, sbml_version)
		MathFormula.readSbml(self, sbml_delay.getMath(), sbml_level, sbml_version)
		MathFormula.setInternalMathFormula(self, MathFormula.ensureFloat(self, MathFormula.getInternalMathFormula(self)))

	def writeSbml(self, sbml_event,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Writes an event priority definition to a sbml file """

		sbml_delay = sbml_event.createDelay()
		SbmlObject.writeSbml(self, sbml_delay, sbml_level, sbml_version)
		sbml_delay.setMath(MathFormula.writeSbml(self, sbml_level, sbml_version))

	def copy(self, obj, symbols_subs={}, conversion_factors={}, time_conversion=None):

		if not self.mathOnly:
			SbmlObject.copy(self, obj)

		t_convs = {}
		for var, conversion in list(conversion_factors.items()):
			t_convs.update({var: var/conversion})

		t_formula = unevaluatedSubs(obj.getInternalMathFormula(rawFormula=False), symbols_subs)
		t_formula = unevaluatedSubs(t_formula, t_convs)

		if time_conversion is not None:
			t_formula *= time_conversion.getInternalMathFormula()

		self.setInternalMathFormula(t_formula)

	def getDefinition(self):
		return self.__definition

	def copySubmodel(self, obj):
		MathFormula.setInternalMathFormula(self, obj.getDeveloppedInternalMathFormula())

	def notZero(self):
		return (
			self.getInternalMathFormula() is not None
			and self.getInternalMathFormula() != SympyInteger(0)
			and self.getInternalMathFormula() != SympyFloat(0.0)
		)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Retryiving GO URI informations

"""


import requests
from json import loads
from lxml import etree

class GoResolver(object):


	# URL = "https://www.ebi.ac.uk/QuickGO-Old/GTerm?id=%s&format=oboxml"
	URL = "http://amigo.geneontology.org/amigo/term/%s/json"
	def __init__(self, go_id):
		self.__id = go_id
		self.__xmlTree = None
		self.__json = None
		self.__name = None
		self.__definition = None

	def lookup(self):

		if self.__id is not None:
			self.getJSON()
			self.parseJSON()

	def getJSON(self):

		r = requests.get(self.URL % self.__id, headers={"Content-Type": "application/json"})
		if r.ok:
			self.__json = loads(r.content.decode('utf-8'))

	def parseJSON(self):

		if self.__json is not None:
			self.__name = self.__json[u'results'][u'name']
			self.__definition = self.__json[u'results'][u'definition']

	# def getXMLTree(self):
	#
	# 	r = requests.get(self.URL % self.__id, headers={"Content-Type": "application/xml"})
	# 	if r.ok:
	# 		self.__xmlTree = etree.fromstring(r.content)
	#
	# def parseXMLTree(self):
	#
	# 	term = self.__xmlTree.find("term")
	# 	self.__name = term.find("name").text
	# 	self.__definition = term.find("def").find("defstr").text

	def getName(self):
		return self.__name

	def getDefinition(self):
		return self.__definition


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.Change import Change
from libsignetsim.sedml.NewXML import NewXML
from libsignetsim.settings.Settings import Settings
from lxml import etree


class AddXML(Change):

	def __init__(self, document):

		Change.__init__(self, document)
		self.__document = document
		self.__newXML = NewXML(document)

	def readSedml(self, change, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		Change.readSedml(self, change, level, version)
		self.__newXML.readSedml(change.getNewXML(), level, version)

	def writeSedml(self, change, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		Change.writeSedml(self, change, level, version)
		change.setNewXML(self.__newXML.writeSedml(level, version))

	def setNewXMLFromString(self, xml_string):
		self.__newXML.setFromString(xml_string)

	def applyChange(self, sbml_tree):

		namespace = {etree.QName(sbml_tree).localname: etree.QName(sbml_tree).namespace}

		if len(sbml_tree.xpath(self.getTarget().getXPath(), namespaces=namespace)) == 0:
			grand_parent = sbml_tree.xpath(self.getTarget().getParentXPath(), namespaces=namespace)[0]
			parent = etree.Element(self.getTarget().getElementTag())
			grand_parent.append(parent)
			new_child = etree.fromstring(self.__newXML.getAsString())
			parent.append(new_child)

		else:
			new_child = etree.fromstring(self.__newXML.getAsString())
			parent = sbml_tree.xpath(self.getTarget().getXPath(), namespaces=namespace)[0]
			parent.append(new_child)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function

from libsignetsim.model.sbml.container.ListOf import ListOf
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.sbml.HasParentObj import HasParentObj
from libsignetsim.model.ModelException import InvalidXPath
from libsignetsim.model.sbml.InitialAssignment import InitialAssignment

from libsignetsim.settings.Settings import Settings
from re import match

class ListOfInitialAssignments(ListOf, SbmlObject, HasParentObj):
	""" Class for the listOfInitialAssignments in a sbml model """

	def __init__(self, model, parent_obj, math_only=False):

		self.__model = model
		ListOf.__init__(self, model)
		HasParentObj.__init__(self, parent_obj)

		# For math submodels, where objects are not sbml objects
		self.mathOnly = math_only
		if not math_only:
			SbmlObject.__init__(self, model)

	def readSbml(self, sbml_list_of_ia,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Reads initial assignments' list from a sbml file """

		for init_ass in sbml_list_of_ia:
			t_init_ass = InitialAssignment(self.__model, self, self.nextId())
			t_init_ass.readSbml(init_ass, sbml_level, sbml_version)
			ListOf.add(self, t_init_ass)

		SbmlObject.readSbml(self, sbml_list_of_ia, sbml_level, sbml_version)

	def writeSbml(self, sbml_model,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Writes initial assignments' list to a sbml file """

		for initial_assignment in self:
			initial_assignment.writeSbml(sbml_model, sbml_level, sbml_version)

		if len(self) > 0:
			SbmlObject.writeSbml(self, sbml_model.getListOfInitialAssignments(), sbml_level, sbml_version)

	def new(self, variable=None, expression=None, rawFormula=False):

		if (variable is not None and expression is not None):
			t_initial_assignment = InitialAssignment(self.__model, self, self.nextId())
			t_initial_assignment.setVariable(variable)
			t_initial_assignment.setPrettyPrintDefinition(expression, rawFormula=rawFormula)
			ListOf.add(self, t_initial_assignment)
			return t_initial_assignment

	def copy(self, obj, deletions=[], sids_subs={}, symbols_subs={}, conversion_factors={}):

		if obj not in deletions:

			SbmlObject.copy(self, obj)

			for init_ass in obj:
				if init_ass not in deletions:

					t_init_ass = InitialAssignment(self.__model, self, self.nextId())
					t_init_ass.copy(init_ass, sids_subs=sids_subs, symbols_subs=symbols_subs, conversion_factors=conversion_factors)
					ListOf.add(self, t_init_ass)

	def copySubmodel(self, obj):

		for init_ass in obj:
			t_init_ass = InitialAssignment(self.__model, self, init_ass.objId, math_only=self.mathOnly)
			t_init_ass.copySubmodel(init_ass)
			ListOf.add(self, t_init_ass)

	def renameSbmlId(self, old_sbml_id, new_sbml_id):

		for obj in self:
			obj.renameSbmlId(old_sbml_id, new_sbml_id)


	def hasInitialAssignment(self, variable):

		for obj in self:
			if obj.getVariable().getSbmlId() == variable.getSbmlId():
				return True

	def containsVariable(self, variable):

		for init_ass in self:
			if init_ass.containsVariable(variable):
				return True
		return False

	def remove(self, init_assignment):
		""" Remove an initial assignment from the list """

		init_assignment.getVariable().unsetRuledBy()
		ListOf.remove(self, init_assignment)

	def removeById(self, obj_id):
		""" Remove an initial assignment from the list """

		self.remove(self.getById(obj_id))

	def resolveXPath(self, selector):

		if not (selector.startswith("initialAssignment") or selector.startswith("sbml:initialAssignment")):
			raise InvalidXPath(selector)

		res_match = match(r'(.*)\[@(.*)=(.*)\]', selector)
		if res_match is None:
			raise InvalidXPath(selector)

		tokens = res_match.groups()
		if len(tokens) != 3:
			raise InvalidXPath(selector)

		object = None
		if tokens[1] == "metaid":
			object = self.getByMetaId(tokens[2][1:-1])

		if object is not None:
			return object

		# If not returned yet
		raise InvalidXPath(selector)

	def getByXPath(self, xpath):
		if len(xpath) > 0:
			return self.resolveXPath(xpath[0]).getByXPath(xpath[1:])
		else:
			return self

	def setByXPath(self, xpath, object):
		self.resolveXPath(xpath[0]).setByXPath(xpath[1:], object)

	def getXPath(self):
		return "/".join([self.getParentObj().getXPath(), "sbml:listOfInitialAssignments"])

	def pprint(self):

		for ode in self:
			ode.pprint()
			print("\n")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function



from sympy import simplify, srepr
from libsignetsim.model.math.sympy_shortcuts import *

from libsignetsim.settings.Settings import Settings
from libsignetsim.model.math.MathException import MathException, DelayNotImplemented

class CMathWriter(object):
	""" Class for handling math formulaes """

	MATH_ERR            =  -1

	MATH_SBML           =   0
	MATH_INTERNAL       =   1
	MATH_DEVINTERNAL    =   2
	MATH_C              =   3
	MATH_PRETTYPRINT    =   4

	MATH_FORMULA        =  20
	MATH_EQUATION       =  21
	MATH_VARIABLE       =  22
	MATH_KINETICLAW     =  23
	MATH_FUNCTION       =  24
	MATH_RATERULE       =  25
	MATH_EVENTASSIGNMENT=  26
	MATH_ASSIGNMENTRULE =  27
	MATH_ALGEBRAICRULE  =  28


	ZERO = SympyInteger(0)

	def __init__(self, model):
		""" Constructor """

		self.model = model

	def writeCCode(self, tree):

		math = self.translateForC(tree)
		if Settings.verbose >= 2:
			print("\n> writeCCode")
			print(">> input : %s" % srepr(tree))
			print(">> input simplified : %s" % str(tree))
			print(">> output : %s" % math)

		return math




	def translateVariableForC(self, variable, derivative=False):
		""" Translates a Sympy symbol in C """

		if str(variable) == "_time_":
			return "t"

		elif str(variable) == "_avogadro_":
			return "RT_NA"

		t_var = None

		if self.model.listOfVariables.containsSymbol(variable):
			t_var = self.model.listOfVariables.getBySymbol(variable)
		else:
			print("> Err : %s" % str(variable))

		t_pos = None
		if t_var.isDerivative():
			if derivative:
				c_var = "ydot"
			else:
				c_var = "y"

			t_pos = t_var.ind+1

		elif t_var.isAssignment():
			c_var = "ass"
			t_pos = t_var.ind+1

		elif t_var.isConstant():
			c_var = "cst"
			t_pos = t_var.ind+1

		elif t_var.isAlgebraic():
			if derivative:
				c_var = "ydot"
			else:
				c_var = "y"

			t_pos = self.model.nbOdes + t_var.ind+1

		else:
			raise MathException("Cannot determine the mathematical type of variable %s" % str(variable))

		return "Ith(%s,%s)" % (c_var, t_pos)


	def translateForC(self, tree):
		""" Translate a sympy tree into a C string """

		if isinstance(tree, int):
			return "RCONST(%d.0)" % tree

		elif isinstance(tree, float):
			t_string = "%.16g" % tree
			if "." not in t_string and "e" not in t_string:
				t_string += ".0"

			return "RCONST(%s)" % t_string

		elif tree.func == SympySymbol:
			return self.translateVariableForC(tree)

		elif tree.func == SympyDerivative:
			return self.translateVariableForC(tree.args[0], derivative=True)

		elif tree.func == SympyInteger:
			return "RCONST(%d.0)" % int(tree)

		elif tree.func == SympyFloat:
			t_string = "%.16g" % float(tree)
			if "." not in t_string and "e" not in t_string:
				t_string += ".0"

			return "RCONST(%s)" % t_string

		elif tree.func == SympyRational:
			return "(%s/%s)" % (self.translateForC(tree.p), self.translateForC(tree.q))

		elif tree.func == SympyNegOne:
			return "RCONST(-1.0)"

		elif tree.func == SympyOne:
			return "RCONST(1.0)"

		elif tree.func == SympyHalf:
			return "RCONST(0.5)"

		elif tree.func == SympyZero:
			return "RCONST(0.0)"

		elif tree == SympyPi:
			return "RT_PI"

		elif tree.func == SympyE or tree.func == SympyExp1:
			return "RT_E"

		elif tree == SympyInf:
			return "RT_INF"

		elif tree == -SympyInf:
			return "-RT_INF"

		elif tree == SympyNan:
			return "RT_NAN"

		elif tree == SympyTrue or tree == True:
			return "1"

		elif tree == SympyFalse or tree == False:
			return "0"

		elif tree.func == SympyMax:
			return "max(%s, %s)" % (
				self.translateForC(tree.args[0]),
				self.translateForC(tree.args[1])
			)


		elif tree.func == SympyAdd:

			t_add = "("
			for i_arg, arg in enumerate(tree.args):
				if i_arg > 0:
					t_add = t_add + " + "
				t_add = t_add + self.translateForC(arg)

			return t_add + ")"


		elif tree.func == SympyMul:

			if len(tree.args) == 2:
				if tree.args[0].func == SympyNegOne:
					return "-" + self.translateForC(tree.args[1])
				if tree.args[1].func == SympyNegOne:
					return "-" + self.translateForC(tree.args[0])


			started = False
			t_minus = ""
			t_mul = ""
			t_divider = ""
			for i_arg, arg in enumerate(tree.args):

				if arg.func == SympyNegOne:
					t_mul = "-" + t_mul

				elif arg.func == SympyPow and arg.args[1].func == SympyNegOne:
					if t_divider == "":
						t_divider = "%s" % self.translateForC(arg.args[0])
					else:
						t_divider += "*%s" % self.translateForC(arg.args[0])
				else:
					if started:
						t_mul += "*"

					started = True
					t_mul += self.translateForC(arg)

			if t_divider == "":
				return t_mul
			else:
				return t_minus + "(" + t_mul + "/(%s))" % t_divider

		# AST_FUNCTION_ABS
		elif tree.func == SympyAbs:
			return "rt_abs(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_QUOTIENT
		elif tree.func == SympyQuotient:
			return "((int) rt_floor(%s/%s))" % (self.translateForC(tree.args[0]), self.translateForC(tree.args[1]))

		# AST_FUNCTION_REM
		elif tree.func == SympyRem:
			return "((int) fmod(%s, %s))" % (self.translateForC(tree.args[0]), self.translateForC(tree.args[1]))


		# AST_FUNCTION_ARCCOS
		elif tree.func == SympyAcos:
			return "rt_acos(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_ARCCOSH
		elif tree.func == SympyAcosh:
			return "rt_acosh(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_ARCCOT
		elif tree.func == SympyAcot:
			return "rt_acot(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_ARCCSC
		elif tree.func == SympyAcsc:
			return "rt_acsc(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_ARCCOTH
		elif tree.func == SympyAcoth:
			return "rt_acoth(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_ARCSIN
		elif tree.func == SympyAsec:
			return "rt_asec(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_ARCSIN
		elif tree.func == SympyAsin:
			return "rt_asin(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_ARCSINH
		elif tree.func == SympyAsinh:
			return "rt_asinh(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_ARCTAN
		elif tree.func == SympyAtan:
			return "rt_atan(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_ARCTANH
		elif tree.func == SympyAtanh:
			return "rt_atanh(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_CEILING
		elif tree.func == SympyCeiling:
			return "rt_ceil(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_COS
		elif tree.func == SympyCos:
			return "rt_cos(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_COSH
		elif tree.func == SympyCosh:
			return "rt_cosh(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_COT
		elif tree.func == SympyCot:
			return "rt_cot(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_COTH
		elif tree.func == SympyCoth:
			return "rt_coth(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_CSC
		elif tree.func == SympyCsc:
			return "rt_csc(%s)" % self.translateForC(tree.args[0])
		# AST_FUNCTION_DELAY
		#TODO
		#SEE BELOW !
		# AST_FUNCTION_EXP
		elif tree.func == SympyExp:
			return "rt_exp(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_FACTORIAL
		elif tree.func == SympyFactorial:
			return "rt_factorial(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_FLOOR
		elif tree.func == SympyFloor:
			return "rt_floor(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_LOG
		elif tree.func == SympyLog:
			if len(tree.args) == 2:
				return "(rt_log(%s)/rt_log(%s))" % (self.translateForC(tree.args[0]), self.translateForC(tree.args[1]))
			else:
				return "rt_log(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_PIECEWISE
		elif tree.func == SympyPiecewise:
			(t_val, t_cond) = tree.args[0]
			line = "(%s?%s" % (self.translateForC(t_cond), self.translateForC(t_val))
			line_end = ")"

			for piece in range(1, len(tree.args)):
				(t_val, t_cond) = tree.args[piece]
				line = line + ":(%s?%s" % (self.translateForC(t_cond), self.translateForC(t_val))
				line_end = line_end + ")"

			line = line + ":(RCONST(0.0))" + line_end
			return line

		# AST_FUNCTION_PIECEWISE
		elif tree.func == SympyITE:
			t_cond = tree.args[0]
			t_val = tree.args[1]
			t_other_val = tree.args[2]
			line = "(%s?%s:%s)" % (self.translateForC(t_cond), self.translateForC(t_val), self.translateForC(t_other_val))

			return line

		# AST_FUNCTION_POWER
		elif tree.func == SympyPow:

			if len(tree.args) == 2 and tree.args[1].func == SympyNegOne:
				return "RCONST(1.0)/(%s)" % self.translateForC(tree.args[0])

			return "rt_pow(%s, %s)" % (self.translateForC(tree.args[0]), self.translateForC(tree.args[1]))

		# AST_FUNCTION_ROOT
		elif tree.func == SympyRoot:
			return "rt_pow(%s,(RCONST(1.0)/%s)" % (self.translateForC(tree.args[0]), self.translateForC(tree.args[1]))

		# AST_FUNCTION_SEC
		elif tree.func == SympySec:
			return "rt_sec(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_SIN
		elif tree.func == SympySin:
			return "rt_sin(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_SINH
		elif tree.func == SympySinh:
			return "rt_sinh(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_TAN
		elif tree.func == SympyTan:
			return "rt_tan(%s)" % self.translateForC(tree.args[0])

		# AST_FUNCTION_TANH
		elif tree.func == SympyTanh:
			return "rt_tanh(%s)" % self.translateForC(tree.args[0])

		elif tree.func == SympyEqual:
			return "rt_eq(%s, %s)" % (self.translateForC(tree.args[0]), self.translateForC(tree.args[1]))

		elif tree.func == SympyUnequal:
			return "rt_neq(%s, %s)" % (self.translateForC(tree.args[0]), self.translateForC(tree.args[1]))

		elif tree.func == SympyGreaterThan:
			return "rt_geq(%s, %s)" % (self.translateForC(tree.args[0]), self.translateForC(tree.args[1]))

		elif tree.func == SympyLessThan:
			return "rt_leq(%s, %s)" % (self.translateForC(tree.args[0]), self.translateForC(tree.args[1]))

		elif tree.func == SympyStrictGreaterThan:
			return "rt_gt(%s, %s)" % (self.translateForC(tree.args[0]), self.translateForC(tree.args[1]))

		elif tree.func == SympyStrictLessThan:
			return "rt_lt(%s, %s)" % (self.translateForC(tree.args[0]), self.translateForC(tree.args[1]))

		elif tree.func == SympyAnd:
			t_args = "("
			for i_arg in range(0, len(tree.args)):
				if i_arg > 0:
					t_args = t_args + " && "
				t_args = t_args + self.translateForC(tree.args[i_arg])
			return t_args + ")"

		elif tree.func == SympyOr:
			t_args = "("
			for i_arg in range(0, len(tree.args)):
				if i_arg > 0:
					t_args = t_args + " || "
				t_args = t_args + self.translateForC(tree.args[i_arg])
			return t_args + ")"

		elif tree.func == SympyXor:
			return self.translateForC(simplify(tree))

		elif tree.func == SympyNot:
			return "(!%s)" % self.translateForC(tree.args[0])

		elif tree.func == SympyImplies:
			# p -> q == !p || q
			# print srepr(tree)
			# print tree.evalf()
			return "(!" + self.translateForC(tree.args[0]) + " || " + self.translateForC(tree.args[1]) + ")"

		elif tree.func == SympyUnevaluatedMin:
			if len(tree.args) == 1:
				return self.translateForC(tree.args[0])

			elif len(tree.args) > 1:
				str = "min(" + self.translateForC(tree.args[0]) + ", " + self.translateForC(tree.args[1]) + ")"

				for i, arg in enumerate(tree.args):
					if i > 1:
						str = "min(" + str + ", " + self.translateForC(tree.args[i]) + ")"
			return str

		elif tree.func == SympyUnevaluatedMax:
			if len(tree.args) == 1:
				return self.translateForC(tree.args[0])

			elif len(tree.args) > 1:
				str = "max(" + self.translateForC(tree.args[0]) + ", " + self.translateForC(tree.args[1]) + ")"

				for i, arg in enumerate(tree.args):
					if i > 1:
						str = "max(" + str + ", " + self.translateForC(tree.args[i]) + ")"
			return str


		elif tree.func == SympyFunction:
			raise DelayNotImplemented()

		else:
			raise MathException("C Math Writer : Unknown Sympy Symbol %s" % str(tree))
			return str(tree)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.Change import Change
from libsignetsim.sedml.container.ListOfParameters import ListOfParameters
from libsignetsim.sedml.container.ListOfVariables import ListOfVariables
from libsignetsim.sedml.math.MathFormula import MathFormula
from libsignetsim.settings.Settings import Settings


class ComputeChange(Change):

	def __init__(self, document):

		Change.__init__(self, document)
		self.__document = document
		self.listOfVariables = ListOfVariables(self.__document, self)
		self.listOfParameters = ListOfParameters(self.__document, self)
		self.__math = MathFormula(self.__document)

	def readSedml(self, change, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		Change.readSedml(self, change, level, version)
		self.listOfVariables.readSedml(change.getListOfVariables(), level, version)
		self.listOfParameters.readSedml(change.getListOfParameters(), level, version)
		self.__math.readSedml(change.getMath(), level, version)

	def writeSedml(self, change, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		Change.writeSedml(self, change, level, version)
		self.listOfVariables.writeSedml(change.getListOfVariables(), level, version)
		self.listOfParameters.writeSedml(change.getListOfParameters(), level, version)
		change.setMath(self.__math.writeSedml(level, version))

	def getMath(self):
		return self.__math

	def setMath(self, math):
		self.__math = math


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function


from libsignetsim.model.math.sympy_shortcuts import SympyEqual, SympyInteger
from libsignetsim.model.math.MathDevelopper import unevaluatedSubs
from sympy import pretty


class ConservationLaw(object):
	""" Sbml model class """

	def __init__ (self, model):
		""" Constructor of model class """

		self.__model = model
		self.LHS = None
		self.RHS = None
		self.vars = []

	def new(self, lhs, rhs, vars):

		self.LHS = lhs
		self.RHS = rhs
		self.vars = vars

	def pprint(self):

		print("%s" % pretty(
			SympyEqual(
				self.LHS.getDeveloppedInternalMathFormula(),
				self.RHS.getDeveloppedInternalMathFormula()
			)
		))


	def __str__(self):
		return ">> %s == %s" % (
			self.LHS.getDeveloppedInternalMathFormula(),
			self.RHS.getDeveloppedInternalMathFormula()
		)


	def getFormula(self, rawFormula=True):

		if not rawFormula:

			comp_symbols = {}
			for comp in self.__model.listOfCompartments:
				comp_symbols.update({comp.symbol.getInternalMathFormula(): SympyInteger(1)})

			return SympyEqual(
				unevaluatedSubs(self.LHS.getInternalMathFormula(), comp_symbols),
				unevaluatedSubs(self.RHS.getInternalMathFormula(), comp_symbols)
			)
		else:
			return SympyEqual(
				self.LHS.getDeveloppedInternalMathFormula(),
				self.RHS.getDeveloppedInternalMathFormula()
			)

	def getNbVars(self):
		return len(self.vars)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from os.path import abspath, join, dirname, expanduser

class Settings(object):

	basePath = abspath(join(dirname(__file__), ".."))

	# Main paths
	# tempDirectory = join(basePath, "tmp/")
	tempDirectory = "/tmp/"

	# C source files
	C_srcPath = join(basePath,"cwriter/src/")
	C_optimizationSrcDirectory = join(C_srcPath,"C_optimization/")
	C_sharedSrcDirectory = join(C_srcPath, "C_shared/")
	C_simulationSrcDirectory = join(C_srcPath, "C_simulation/")

	# C destination paths
	C_optimizationDirectory = "C_optimization/"
	C_sharedDirectory = "C_shared/"
	C_simulationDirectory = "C_simulation/"
	C_simulationResultsDirectory = "results/"
	C_generatedDirectory = "C_generated/"

	# let's remake all this C links
	# C source files
	C_srcPath_v2 = join(basePath,"lib/plsa")
	C_generatedDirectory_v2 = "src/"

	# C destination paths
	C_optimizationDirectory_v2 = "plsa/"


	# SBML settings
	defaultSbmlLevel = 3
	defaultSbmlVersion = 1

	# SED-ML settings
	defaultSedmlLevel = 1
	defaultSedmlVersion = 2

	# NuML settings
	defaultNuMLLevel = 1
	defaultNuMLVersion = 1

	# Test settings
	defaultTestAbsTol = 1e-6
	defaultTestRelTol = 1e-3

	defaultAbsTol = 1e-12
	defaultRelTol = 1e-10

	simulationTimeMin = 0
	simulationTimeMax = 4000
	simulationLogScale = False
	simulationTimeEch = None
	simulationNbSamples = 101
	simulationKeepFiles = True

	steadyStatesMaxTime = 100000
	defaultMaxProcNumbers = 2

	plotTimeMin = 0
	plotTimeEch = 10
	plotTimeMax = 100
	plotTitle = ""

	optimizationDefaultConstantLowerBound = 1e-8
	optimizationDefaultConstantUpperBound = 1e+8

	optimizationDefaultParamLowerBound = 1e-8
	optimizationDefaultParamUpperBound = 1e+8

	optimizationDefaultInitialValueLowerBound = 1e-8
	optimizationDefaultInitialValueUpperBound = 1e+8

	optimizationDefaultCompartmentLowerBound = 1e-8
	optimizationDefaultCompartmentUpperBound = 1e+8

	verbose = 0
	verboseTiming = 0
	showSbmlErrors = False
	reduceByDefault = False


	sbmlTestCasesPath = join('/', 'tmp',"sbml-test-suite", "cases", "semantic")
	sbmlTestResultsPath = join('/', 'tmp',"test-suite-results")

	defaultCVODEmaxNumSteps = 5000
	defaultCVODEMaxConvFails = 100
	defaultCVODEMaxErrFails = 70

	defaultScoreNegativePenalty = 0

	defaultPlsaSeed = 0
	defaultPlsaInitialTemperature = 1
	defaultPlsaGainForJumpSizeControl = 5
	defaultPlsaInterval = 100
	# For enzyme, 0.005 is very good (95% of good results)
	# you can also use 0.01, but that's like 80% good results
	defaultPlsaLambda = 0.001
	defaultPlsaLambdaMemU = 200
	defaultPlsaLambdaMemV = 1000
	defaultPlsaControl = 1
	defaultPlsaInitialMoves = 2000
	defaultPlsaTau = 1000
	defaultPlsaFreezeCount = 100
	defaultPlsaUpdateSSkip = 1
	defaultPlsaCriterion = 0.001
	defaultPlsaPrecision = 7
	defaultPlsaMixInterval = 10
	defaultPlsaDistribution = 1
	defaultPlsaQ = 1.0

	defaultPlsaTraceLog = 0
	defaultPlsaParamsLog = 0

	color_scheme = (["#FFB300", "#803E75", "#FF6800", "#A6BDD7"]
					+ ["#C10020", "#CEA262", "#817066", "#007D34"]
					+ ["#F6768E", "#00538A", "#FF7A5C", "#53377A"]
					+ ["#FF8E00", "#B32851", "#F4C800", "#7F180D"]
					+ ["#93AA00", "#593315", "#F13A13", "#232C16"])

	color_scheme_light = ['#67b6ce']


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Testing the research for conservation laws, and the subsequent reduction

"""

from libsignetsim import Model, KineticLaw, MathFormula

from libsignetsim.model.math.sympy_shortcuts import SympyEqual, SympyZero
from unittest import TestCase
from sympy import simplify


class TestModelDefinition(TestCase):
	""" Tests high level functions """


	def testReactions(self):
		""" First three test to check the definition of mass action, michaelis and hill kinetics, the last one
			just to check mass action for species defined as substance. All check the option rawFormula, which 
		"""

		model = Model()

		c = model.listOfCompartments.new("c", value=2)

		s1 = model.listOfSpecies.new("s1")
		s2 = model.listOfSpecies.new("s2")
		s3 = model.listOfSpecies.new("s3")
		s4 = model.listOfSpecies.new("s4", hasOnlySubstanceUnits=True)
		s5 = model.listOfSpecies.new("s5", hasOnlySubstanceUnits=True)
		s6 = model.listOfSpecies.new("s6", hasOnlySubstanceUnits=True)

		p1 = model.listOfParameters.new("p1")
		p2 = model.listOfParameters.new("p2")
		p3 = model.listOfParameters.new("p3")

		r1 = model.listOfReactions.new("reaction 1")
		r1.listOfReactants.add(s1)
		r1.listOfReactants.add(s2)
		r1.listOfProducts.add(s3)
		r1.setKineticLaw(KineticLaw.MASS_ACTION, reversible=True, parameters=[p1,p2])

		formula_r1 = MathFormula(model)
		formula_r1.setPrettyPrintMathFormula("p1*s1*s2 - p2*s3", rawFormula=True)

		self.assertTrue(self.sympyEqual(
			r1.kineticLaw.getDefinition().getDeveloppedInternalMathFormula(),
			formula_r1.getDeveloppedInternalMathFormula()
		))

		formula_r1_amount = MathFormula(model)
		formula_r1_amount.setPrettyPrintMathFormula("c*(p1*s1/c*s2/c - p2*s3/c)", rawFormula=True)

		self.assertTrue(self.sympyEqual(
			r1.kineticLaw.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula(),
			formula_r1_amount.getDeveloppedInternalMathFormula()
		))


		r2 = model.listOfReactions.new("reaction 2")
		r2.listOfReactants.add(s1)
		r2.listOfModifiers.add(s2)
		r2.listOfProducts.add(s3)
		r2.setKineticLaw(KineticLaw.MICHAELIS, reversible=False, parameters=[p1, p2])

		formula_r2 = MathFormula(model)
		formula_r2.setPrettyPrintMathFormula("p1*s1*s2/(p2+s1)", rawFormula=True)
		self.assertTrue(self.sympyEqual(
			r2.kineticLaw.getDefinition().getDeveloppedInternalMathFormula(),
			formula_r2.getDeveloppedInternalMathFormula()
		))

		formula_r2_amount = MathFormula(model)
		formula_r2_amount.setPrettyPrintMathFormula("c*(p1*s1/c*s2/c/(p2+s1/c))", rawFormula=True)
		self.assertTrue(self.sympyEqual(
			r2.kineticLaw.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula(),
			formula_r2_amount.getDeveloppedInternalMathFormula()
		))

		# Reaction 3 : Hill equation, species declared as concentration
		r3 = model.listOfReactions.new("reaction 3")
		r3.listOfReactants.add(s1)
		r3.listOfModifiers.add(s2)
		r3.listOfProducts.add(s3)
		r3.setKineticLaw(KineticLaw.HILL, reversible=False, parameters=[p1, p2, p3])

		formula_r3 = MathFormula(model)
		formula_r3.setPrettyPrintMathFormula("p1*s1^p3*s2/(p2+s1^p3)", rawFormula=True)
		self.assertTrue(self.sympyEqual(
			r3.kineticLaw.getDefinition().getDeveloppedInternalMathFormula(),
			formula_r3.getDeveloppedInternalMathFormula()
		))

		formula_r3_amount = MathFormula(model)
		formula_r3_amount.setPrettyPrintMathFormula("c*(p1*(s1/c)^p3*(s2/c)/(p2+(s1/c)^p3))", rawFormula=True)
		self.assertTrue(self.sympyEqual(
			r3.kineticLaw.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula(),
			formula_r3_amount.getDeveloppedInternalMathFormula()
		))

		# Reaction 4 : mass action with species declared as substance
		r4 = model.listOfReactions.new("reaction 4")
		r4.listOfReactants.add(s4)
		r4.listOfReactants.add(s5)
		r4.listOfProducts.add(s6)
		r4.setKineticLaw(KineticLaw.MASS_ACTION, reversible=True, parameters=[p1, p2])

		formula_r4 = MathFormula(model)
		formula_r4.setPrettyPrintMathFormula("(p1*s4*s5 - p2*s6)", rawFormula=True)

		self.assertTrue(self.sympyEqual(
			r4.kineticLaw.getDefinition().getDeveloppedInternalMathFormula(),
			formula_r4.getDeveloppedInternalMathFormula()
		))

		self.assertTrue(self.sympyEqual(
			r4.kineticLaw.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula(),
			formula_r4.getDeveloppedInternalMathFormula()
		))

	def testAssignmentRule(self):
		model = Model()

		c = model.listOfCompartments.new("c", value=2)

		s1 = model.listOfSpecies.new("s1")
		s2 = model.listOfSpecies.new("s2")
		s3 = model.listOfSpecies.new("s3", hasOnlySubstanceUnits=True)
		s4 = model.listOfSpecies.new("s4", hasOnlySubstanceUnits=True)

		p1 = model.listOfParameters.new("p1")

		a1 = model.listOfRules.newAssignmentRule(s2, "p1*s1")

		f1 = MathFormula(model)
		f1.setPrettyPrintMathFormula("p1*s1", rawFormula=True)

		self.assertTrue(self.sympyEqual(
			a1.getDefinition().getDeveloppedInternalMathFormula(),
			f1.getDeveloppedInternalMathFormula()
		))

		f2 = MathFormula(model)
		f2.setPrettyPrintMathFormula("c*(p1/c)*(s1/c)", rawFormula=True)

		self.assertTrue(self.sympyEqual(
			a1.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula(),
			f1.getDeveloppedInternalMathFormula()
		))

		a2 = model.listOfRules.newAssignmentRule(s4, "p1*s3")

		f3 = MathFormula(model)
		f3.setPrettyPrintMathFormula("p1*s3", rawFormula=True)

		self.assertTrue(self.sympyEqual(
			a2.getDefinition().getDeveloppedInternalMathFormula(),
			f3.getDeveloppedInternalMathFormula()
		))

		self.assertTrue(self.sympyEqual(
			a2.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula(),
			f3.getDeveloppedInternalMathFormula()
		))


	def testRateRule(self):
		model = Model()

		c = model.listOfCompartments.new("c", value=2)

		s1 = model.listOfSpecies.new("s1")
		s2 = model.listOfSpecies.new("s2")
		s3 = model.listOfSpecies.new("s3", hasOnlySubstanceUnits=True)
		s4 = model.listOfSpecies.new("s4", hasOnlySubstanceUnits=True)

		p1 = model.listOfParameters.new("p1")

		r1 = model.listOfRules.newRateRule(s2, "p1*s1")

		f1 = MathFormula(model)
		f1.setPrettyPrintMathFormula("p1*s1", rawFormula=True)

		self.assertTrue(self.sympyEqual(
			r1.getDefinition().getDeveloppedInternalMathFormula(),
			f1.getDeveloppedInternalMathFormula()
		))

		f2 = MathFormula(model)
		f2.setPrettyPrintMathFormula("c*(p1/c)*(s1/c)", rawFormula=True)

		self.assertTrue(self.sympyEqual(
			r1.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula(),
			f1.getDeveloppedInternalMathFormula()
		))

		r2 = model.listOfRules.newRateRule(s4, "p1*s3")

		f3 = MathFormula(model)
		f3.setPrettyPrintMathFormula("p1*s3", rawFormula=True)

		self.assertTrue(self.sympyEqual(
			r2.getDefinition().getDeveloppedInternalMathFormula(),
			f3.getDeveloppedInternalMathFormula()
		))

		self.assertTrue(self.sympyEqual(
			r2.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula(),
			f3.getDeveloppedInternalMathFormula()
		))


	def testAlgebraicRule(self):
		model = Model()

		c = model.listOfCompartments.new("c", value=2)

		s1 = model.listOfSpecies.new("s1")
		s2 = model.listOfSpecies.new("s2")
		s3 = model.listOfSpecies.new("s3", hasOnlySubstanceUnits=True)
		s4 = model.listOfSpecies.new("s4", hasOnlySubstanceUnits=True)

		p1 = model.listOfParameters.new("p1")

		a1 = model.listOfRules.newAlgebraicRule("s2 - p1*s1")

		f1 = MathFormula(model)
		f1.setPrettyPrintMathFormula("s2 - p1*s1", rawFormula=True)

		self.assertTrue(self.sympyEqual(
			a1.getDefinition().getDeveloppedInternalMathFormula(),
			f1.getDeveloppedInternalMathFormula()
		))

		f2 = MathFormula(model)
		f2.setPrettyPrintMathFormula("s2/c - p1*(s1/c)", rawFormula=True)

		self.assertTrue(self.sympyEqual(
			a1.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula(),
			f2.getDeveloppedInternalMathFormula()
		))

		a2 = model.listOfRules.newAlgebraicRule("s3 - p1*s4")

		f3 = MathFormula(model)
		f3.setPrettyPrintMathFormula("s3 - p1*s4", rawFormula=True)

		self.assertTrue(self.sympyEqual(
			a2.getDefinition().getDeveloppedInternalMathFormula(),
			f3.getDeveloppedInternalMathFormula()
		))

		self.assertTrue(self.sympyEqual(
			a2.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula(),
			f3.getDeveloppedInternalMathFormula()
		))


	def testInitialAssignment(self):
		model = Model()

		c = model.listOfCompartments.new("c", value=2)

		s1 = model.listOfSpecies.new("s1")
		s2 = model.listOfSpecies.new("s2")
		s3 = model.listOfSpecies.new("s3", hasOnlySubstanceUnits=True)
		s4 = model.listOfSpecies.new("s4", hasOnlySubstanceUnits=True)

		p1 = model.listOfParameters.new("p1")

		i1 = model.listOfInitialAssignments.new(s2, "p1*s1")

		f1 = MathFormula(model)
		f1.setPrettyPrintMathFormula("p1*s1", rawFormula=True)

		self.assertTrue(self.sympyEqual(
			i1.getDefinition().getDeveloppedInternalMathFormula(),
			f1.getDeveloppedInternalMathFormula()
		))

		f2 = MathFormula(model)
		f2.setPrettyPrintMathFormula("c*(p1/c)*(s1/c)", rawFormula=True)

		self.assertTrue(self.sympyEqual(
			i1.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula(),
			f1.getDeveloppedInternalMathFormula()
		))

		i2 = model.listOfInitialAssignments.new(s4, "p1*s3")

		f3 = MathFormula(model)
		f3.setPrettyPrintMathFormula("p1*s3", rawFormula=True)

		self.assertTrue(self.sympyEqual(
			i2.getDefinition().getDeveloppedInternalMathFormula(),
			f3.getDeveloppedInternalMathFormula()
		))

		self.assertTrue(self.sympyEqual(
			i2.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula(),
			f3.getDeveloppedInternalMathFormula()
		))


	def sympyEqual(self, a, b):

		# print "test %s == %s : %s" % (simplify(a), simplify(b), (simplify(a-b) == 0))
		# print "test %s == %s : %s" % (a, b, (simplify(a-b) == 0))
		return simplify(a-b) == 0


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.math.MathVariable import MathVariable
from libsignetsim.model.sbml.SbmlVariable import SbmlVariable
from libsignetsim.settings.Settings import Settings
from libsignetsim.model.ModelException import SbmlException

class Variable(SbmlVariable, MathVariable):

	def __init__(self, model, sbml_type, is_from_reaction=None):

		self.__model = model

		SbmlVariable.__init__(self, model, sbml_type, is_from_reaction)
		MathVariable.__init__(self, model, is_from_reaction)

	def new(self, string, sbml_type=SbmlVariable.PARAMETER):
		SbmlVariable.new(self, string, sbml_type)
		MathVariable.new(self, string)

	def copy(self, obj, sids_subs={}, symbols_subs={}, conversion_factor=None):
		SbmlVariable.copy(self, obj, sids_subs=sids_subs)
		MathVariable.copy(self, obj, symbols_subs=symbols_subs, conversion_factor=conversion_factor)

	def readSbml(self, sbml_variable,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		SbmlVariable.readSbml(self, sbml_variable, sbml_level, sbml_version)
		MathVariable.readSbml(self, sbml_variable, sbml_level, sbml_version)

	def writeSbml(self, sbml_variable,
						sbml_level=Settings.defaultSbmlLevel,
						sbml_version=Settings.defaultSbmlVersion):

		SbmlVariable.writeSbml(self, sbml_variable, sbml_level, sbml_version)
		MathVariable.writeSbml(self, sbml_variable, sbml_level, sbml_version)

	def setSbmlId(self, sbml_id, prefix="", model_wide=True):

		if sbml_id is not None:
			t_sbml_id = prefix + sbml_id
			if t_sbml_id == self.getSbmlId():
				pass

			elif not self.__model.listOfVariables.containsSbmlId(t_sbml_id) or (self.isParameter() and self.isLocalParameter()):
				SbmlVariable.setSbmlId(self, t_sbml_id, prefix, model_wide)
				MathVariable.setSbmlId(self, t_sbml_id, prefix)

			else:
				raise SbmlException("Identifier %s already exist" % sbml_id)

	def renameSbmlId(self, old_sbml_id, new_sbml_id):
		""" Function to rename a sbml id which could be countained
			in the variable, aka in the value
		"""
		SbmlVariable.renameSbmlId(self, new_sbml_id)
		MathVariable.renameSbmlId(self, old_sbml_id, new_sbml_id)

	def renameSymbol(self, old_sbml_id, new_sbml_id):
		SbmlVariable.renameSbmlId(self, new_sbml_id)
		MathVariable.renameSymbol(self, old_sbml_id, new_sbml_id)

	def isInAlgebraicRules(self):
		return self.__model.listOfRules.algebraicContainsVariable(self)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.SedBase import SedBase
from libsignetsim.settings.Settings import Settings

class AlgorithmParameter(SedBase):

	REL_TOL = 'KISAO:0000209'
	ABS_TOL = 'KISAO:0000211'

	def __init__(self, document):

		SedBase.__init__(self, document)

		self.__document = document
		self.__kisaoID = None
		self.__value = None

	def readSedml(self, algo_param, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		SedBase.readSedml(self, algo_param, level, version)

		if algo_param.isSetKisaoID():
			self.__kisaoID = algo_param.getKisaoID()

		if algo_param.isSetValue():
			self.__value = algo_param.getValue()

	def writeSedml(self, algo_param, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		SedBase.writeSedml(self, algo_param, level, version)

		if self.__kisaoID is not None:
			algo_param.setKisaoID(self.__kisaoID)

		if self.__value is not None:
			algo_param.setValue(str(self.__value))

	def isRelTol(self):
		return self.__kisaoID == self.REL_TOL

	def isAbsTol(self):
		return self.__kisaoID == self.ABS_TOL

	def getValue(self):
		return self.__value

	def setRelTol(self, rel_tol):
		self.__kisaoID = self.REL_TOL
		self.__value = rel_tol

	def setAbsTol(self, abs_tol):
		self.__kisaoID = self.ABS_TOL
		self.__value = abs_tol


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.HasId import HasId
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.sbml.Unit import Unit
from libsignetsim.settings.Settings import Settings

from libsbml import UNIT_KIND_AMPERE, UNIT_KIND_AVOGADRO, UNIT_KIND_BECQUEREL,\
	UNIT_KIND_CANDELA, UNIT_KIND_COULOMB, UNIT_KIND_DIMENSIONLESS,\
	UNIT_KIND_FARAD, UNIT_KIND_GRAM, UNIT_KIND_GRAY, UNIT_KIND_HENRY,\
	UNIT_KIND_HERTZ, UNIT_KIND_ITEM, UNIT_KIND_JOULE, UNIT_KIND_KATAL,\
	UNIT_KIND_KELVIN, UNIT_KIND_KILOGRAM, UNIT_KIND_LITRE, UNIT_KIND_LUMEN,\
	UNIT_KIND_LUX, UNIT_KIND_METRE, UNIT_KIND_MOLE, UNIT_KIND_NEWTON,\
	UNIT_KIND_OHM, UNIT_KIND_PASCAL, UNIT_KIND_RADIAN, UNIT_KIND_SECOND,\
	UNIT_KIND_SIEMENS, UNIT_KIND_SIEVERT, UNIT_KIND_STERADIAN, UNIT_KIND_TESLA,\
	UNIT_KIND_VOLT, UNIT_KIND_WATT, UNIT_KIND_WEBER, UNIT_KIND_INVALID

class UnitDefinition(HasId, SbmlObject):


	def __init__ (self, model, objId=0):

		self.__model = model
		self.objId = objId

		SbmlObject.__init__(self, model)
		HasId.__init__(self, model)

		self.listOfUnits = []

	def defaultConcentrationUnit(self):

		HasId.setName(self, "nanomolars")
		HasId.setSbmlId(self, "nanomolars")

		t_unit = Unit(self.__model, len(self.listOfUnits))
		t_unit.new(UNIT_KIND_MOLE, 1, -9)

		self.listOfUnits.append(t_unit)

		t_unit = Unit(self.__model, len(self.listOfUnits))
		t_unit.new(UNIT_KIND_LITRE, -1)

		self.listOfUnits.append(t_unit)

	def defaultAmountUnit(self):

		HasId.setName(self, "nanomoles")
		HasId.setSbmlId(self, "nanomoles")

		t_unit = Unit(self.__model, len(self.listOfUnits))
		t_unit.new(UNIT_KIND_MOLE, 1, -9)

		self.listOfUnits.append(t_unit)


	def defaultTimeUnits(self):

		HasId.setName(self, "seconds")
		HasId.setSbmlId(self, "seconds")

		t_unit = Unit(self.__model, len(self.listOfUnits))
		t_unit.new(UNIT_KIND_SECOND)

		self.listOfUnits.append(t_unit)

	def defaultCompartmentUnits(self):

		HasId.setName(self, "litres")
		HasId.setSbmlId(self, "litres")

		t_unit = Unit(self.__model, len(self.listOfUnits))
		t_unit.new(UNIT_KIND_LITRE)

		self.listOfUnits.append(t_unit)



	def isEqual(self, other_units):

		if len(self.listOfUnits) == len(other_units.listOfUnits):
			found = False
			for i, t_unit in enumerate(self.listOfUnits):
				found = False

				for t_other_unit in other_units.listOfUnits:

					if t_unit.isEqual(t_other_unit):
						found = True
				if found == False:
					return False
			if found == True:

				return True
			else:

				return False
		else:
			return False


	def copy(self, obj, usids_subs={}):

		HasId.copy(self, obj, sids_subs=usids_subs)

		for unit in obj.listOfUnits:
			t_unit = Unit(self.__model, len(self.listOfUnits))
			t_unit.copy(unit)
			self.listOfUnits.append(t_unit)


	def readSbml(self, unitDefinition, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		SbmlObject.readSbml(self, unitDefinition, sbml_level, sbml_version)
		HasId.readSbml(self, unitDefinition, sbml_level, sbml_version)

		for unit in unitDefinition.getListOfUnits():
			t_unit = Unit(self.__model, len(self.listOfUnits))
			t_unit.readSbml(unit, sbml_level, sbml_version)
			self.listOfUnits.append(t_unit)


	def writeSbml(self, sbml_model, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		t_unitDefinition = sbml_model.createUnitDefinition()
		SbmlObject.writeSbml(self, t_unitDefinition, sbml_level, sbml_version)
		HasId.writeSbml(self, t_unitDefinition, sbml_level, sbml_version)

		for unit in self.listOfUnits:
			unit.writeSbml(t_unitDefinition, sbml_level, sbml_version)


	def __str__(self):

		return self.getName() + " (" + self.printUnitDefinition() + ")"

	def printUnitDefinition(self):

		if len(self.listOfUnits) > 0:
			t_unit = ""
			i_unit = 0
			for unit in self.listOfUnits:
				if unit.exponent > 0:
					if i_unit > 0:
						t_unit += "."
					t_unit += str(unit)
					i_unit += 1

			for unit in self.listOfUnits:
				if unit.exponent < 0:
					if i_unit > 0:
						t_unit += "."
					t_unit += str(unit)
					i_unit += 1

			return t_unit
		else:
			return "UNDEFINED"



	def newUnit(self):
		self.listOfUnits.append(Unit(self.__model, len(self.listOfUnits)))
		return self.listOfUnits[len(self.listOfUnits)-1]

	def deleteUnit(self, objId):
		self.listOfUnits.remove(self.listOfUnits[objId])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Initialization of the combine archive module

"""


from libsignetsim.settings.Settings import Settings

from lxml import etree

class Manifest(object):

	OMEX_MANIFEST = "{http://identifiers.org/combine.specifications/omex-manifest}"
	CONTENT = OMEX_MANIFEST + "content"
	LOCATION = "location"
	FORMAT = "format"
	MASTER = "master"

	def __init__(self, archive):

		self.__archive = archive
		self.__locations = []
		self.__formats = []
		self.__masters = []

	def readManifest(self, manifest):

		root = etree.fromstring(manifest)
		for child in root:
			if child.tag == self.CONTENT:
				if self.LOCATION in list(child.keys()) and self.FORMAT in list(child.keys()):

					self.__locations.append(child.get(self.LOCATION)[2:])

					# t_format = child.get(self.FORMAT).split("/")
					self.__formats.append(child.get(self.FORMAT))

					if child.get(self.MASTER) is not None:
						self.__masters.append(child.get(self.MASTER) == "true")
					else:
						self.__masters.append(False)


	def writeManifest(self):

		root = etree.Element("omexManifest")
		root.set("xmlns", "http://identifiers.org/combine.specifications/omex-manifest")

		archive = etree.SubElement(root, "content")
		archive.set("location", ".")
		archive.set("format", "http://identifiers.org/combine.specifications/omex")

		manifest = etree.SubElement(root, "content")
		manifest.set("location", "./manifest.xml")
		manifest.set("format", "http://identifiers.org/combine.specifications/omex-manifest")

		for file in self.__archive.getListOfFiles():
			t_content = etree.SubElement(root, "content")
			t_content.set("location", "./" + file.getFilename())
			t_content.set("format", file.getFormat())
			if file.isMaster():
				t_content.set("master", "true")

		tree = etree.tostring(root, encoding='utf-8', pretty_print=True, xml_declaration=True)
		return str(tree.decode('utf-8'))

	# def getMaster(self):
	# 	print self.__masters.index(True)
	# 	if True in self.__masters:
	# 		return self.__locations[self.__masters.index(True)]

	def isInManifest(self, filename):
		return filename in self.__locations

	def addInManifest(self, filename, format, master=False):
		self.__locations.append(filename)
		self.__formats.append(format)
		self.__masters.append(master)

	def getFormat(self, filename):
		return self.__formats[self.__locations.index(filename)]

	def setFormat(self, filename, format):
		self.__formats[self.__locations.index(filename)] = format


	def isMaster(self, filename):
		return self.__masters[self.__locations.index(filename)]

	def setMaster(self, filename):
		# Commented for now since we can actually have several masters
		# if True in self.__masters:
		# 	self.__masters = [False]*len(self.__locations)

		self.__masters[self.__locations.index(filename)] = True




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function


from libsignetsim.simulation.CWriterSimulation import CWriterSimulation
from libsignetsim.simulation.SimulationException import (
	SimulationExecutionException, SimulationCompilationException, SimulationNoDataException, SimulationException
)
from libsignetsim.model.Model import Model
from libsignetsim.settings.Settings import Settings
from time import time, clock
from os import mkdir, setpgrp, getcwd
from os.path import join, isfile, exists, getsize
from subprocess import call
from shutil import rmtree
from random import choice
from string import ascii_uppercase, ascii_lowercase, digits

class Simulation(CWriterSimulation):

	SIM_DONE            =   10
	SIM_TODO            =   11


	def __init__ (self,
					list_of_models,
				  	time_min,
					list_samples,
					experiment=None,
					abs_tol=Settings.defaultAbsTol,
					rel_tol=Settings.defaultRelTol,
					keep_files=Settings.simulationKeepFiles):

		self.keepFiles = keep_files
		self.sessionId = None
		self.simulationId = ''.join(choice(ascii_uppercase + ascii_lowercase + digits) for _ in range(12))


		if isinstance(list_of_models, list) and isinstance(list_of_models[0], Model):
			self.listOfModels = list_of_models
		elif isinstance(list_of_models, Model):
			self.listOfModels = [list_of_models]
		else:
			raise SimulationException("Unknown format for models")

		if isinstance(list_samples, list) and isinstance(list_samples[0], list):
			self.listSamples = list_samples

		elif isinstance(list_samples, list) and isinstance(list_samples[0], float):
			self.listSamples = [list_samples]*len(self.listOfModels)

		elif isinstance(list_samples, list) and isinstance(list_samples[0], int):
			self.listSamples = [list_samples]*len(self.listOfModels)

		else:
			raise SimulationException("Unknown format for list of samples")

		if time_min is not None:
			if isinstance(time_min, list):
				self.timeMin = time_min

			elif isinstance(time_min, float):
				self.timeMin = [time_min]*len(self.listOfModels)

			elif isinstance(time_min, int):
				self.timeMin = [time_min]*len(self.listOfModels)

			else:
				raise SimulationException("Unknown format for minimum time")
		else:
			self.timeMin = [Settings.simulationTimeMin]*len(self.listOfModels)

		if abs_tol is not None:
			if isinstance(abs_tol, list):
				self.absTol = abs_tol

			elif isinstance(abs_tol, float):
				self.absTol = [abs_tol]*len(self.listOfModels)

			elif isinstance(abs_tol, int):
				self.absTol = [abs_tol]*len(self.listOfModels)

			else:
				raise SimulationException("Unknown format for absolute tolerance")
		else:
			self.absTol = [Settings.defaultAbsTol]*len(self.listOfModels)

		if rel_tol is not None:
			if isinstance(rel_tol, list):
				self.relRol = rel_tol

			elif isinstance(rel_tol, float):
				self.relTol = [rel_tol]*len(self.listOfModels)

			elif isinstance(rel_tol, int):
				self.relTol = [rel_tol] * len(self.listOfModels)

			else:
				raise SimulationException("Unknown format for relative tolerance")
		else:
			self.relTol = [Settings.defaultRelTol]*len(self.listOfModels)

		CWriterSimulation.__init__(self,
						list_of_models=self.listOfModels,
						time_min=self.timeMin,
						list_samples=self.listSamples,
						experiment=experiment,
						abs_tol=self.absTol,
						rel_tol=self.relTol)

		self.listOfModels = list_of_models

		self.__simulationDone = self.SIM_TODO
		self.rawData = None

		self.nbConditions = 0

		if experiment is not None:
			self.nbConditions = len(experiment.listOfConditions)


	def getTempDirectory(self):
		if self.sessionId is None:
			return join(Settings.tempDirectory,
								"simulation_%s/" % self.simulationId)
		else:
			return join(self.sessionId,
								("simulation_%s/" % self.simulationId))


	def isDone(self):
		return self.__simulationDone == self.SIM_DONE


	def cleanTempDirectory(self):
		rmtree(self.getTempDirectory())


	def writeSimulationFiles(self):

		mkdir(self.getTempDirectory())
		CWriterSimulation.writeSimulationFiles(self)

		res_path = join(self.getTempDirectory(), Settings.C_simulationResultsDirectory)

		if not exists(res_path):
			mkdir(res_path)

	def __compile__(self, nb_procs=4):


		if self.nbConditions == 0 or nb_procs <= 1:
			cmd_comp = "make -C %s sim-serial" % self.getTempDirectory()
		else:
			cmd_comp = "make -C %s sim-parallel" % self.getTempDirectory()


		with open("%sout_comp" % self.getTempDirectory(), "w") as stdout, open("%serr_comp" % self.getTempDirectory(), "w") as stderr:
			res_comp = call(cmd_comp,
							stdout=stdout, #open("%sout_comp" % self.getTempDirectory(),"w"),
							stderr=stderr, #open("%serr_comp" % self.getTempDirectory(),"w"),
							shell=True,preexec_fn=setpgrp,close_fds=True)

		if (res_comp != 0
			or getsize(self.getTempDirectory() + "err_comp") > 0):


			if Settings.verbose >= 1:
				print("-"*40 + "\n")
				print("> Error during simulation compilation :")
				with open(self.getTempDirectory() + "err_comp", 'r') as f_err_comp:
					for line in f_err_comp:
						print(line)

				print("-"*40 + "\n")
				# f_err_comp.close()

			raise SimulationCompilationException("Error during simulation compilation (%d)" % res_comp)

	def __execute__(self, nb_procs=Settings.defaultMaxProcNumbers, steady_states=False, timeout=None):

		present_dir = getcwd()

		flags = ""
		if steady_states:
			flags += "-s "

		cmd_sim = ""
		if self.nbConditions == 0 or nb_procs <= 1:
			cmd_sim = "cd %s; ./sim-serial %s; cd %s" % (
					self.getTempDirectory(),
					flags, present_dir)

		else:
			cmd_sim = "cd %s; mpirun -np %d ./sim-parallel %s; cd %s" % (
					self.getTempDirectory(),
					nb_procs, flags,
					present_dir)

		with open("%sout_sim" % self.getTempDirectory(), "w") as stdout, open("%serr_sim" % self.getTempDirectory(), "w") as stderr:

			res_sim = call(
				cmd_sim, stdout=stdout, stderr=stderr,
				shell=True, preexec_fn=setpgrp, close_fds=True
			)

		if res_sim != 0 or getsize(join(self.getTempDirectory(), "err_sim")) > 0:

			# There is some weird error here, apparently caused by docker filesystem.
			# ** apparently ** we can ignore it
			non_docker_err = False
			with open(self.getTempDirectory() + "err_sim", 'r') as f_err_sim:
				for line in f_err_sim:
					if not line.startswith("Unexpected end of /proc/mounts"):
						non_docker_err = True


			if non_docker_err:
				if Settings.verbose >= 1:
					print("-" * 40 + "\n")
					print("> Error during simulation execution :")
					with open(self.getTempDirectory() + "err_sim", 'r') as f_err_sim:
						for line in f_err_sim:
							if not line.startswith("Unexpected end of /proc/mounts"):
								print(line)
					print("-" * 40 + "\n")
					# f_err_sim.close()

				raise SimulationExecutionException("Error during simulation execution (%d)" % res_sim)

		else:
			if Settings.verbose >= 2:
				print("-"*40 + "\n")
				print("> Execution returned : \n")
				with open(self.getTempDirectory() + "out_sim", 'r') as f_out_sim:
					for line in f_out_sim:
						print(line)

				print("-"*40 + "\n")
				# f_out_sim.close()


	def runSimulation(self, progress_signal=None, steady_states=False, nb_procs=Settings.defaultMaxProcNumbers, timeout=None):

		start = time()
		self.__compile__(nb_procs=nb_procs)

		mid = time()

		if Settings.verboseTiming >= 1:
			print(">> Compilation executed in %.2fs" % (mid-start))

		self.__execute__(nb_procs=nb_procs, steady_states=steady_states)
		end = time()

		if Settings.verboseTiming >= 1:
			print(">> Simulation executed in %.2fs" % (end-mid))

		self.__simulationDone = self.SIM_DONE
		self.__simulationDuration = (end - start)*min(self.nbConditions * len(self.listOfModels), nb_procs)

	def getRawData(self):
		if self.__simulationDone == self.SIM_DONE:
			return self.rawData
		else:
			raise SimulationNoDataException("No data : simulation hasn't been executed yet")

	def getSimulationDuration(self):
		return self.__simulationDuration



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import division

from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.ModelException import SbmlException
from libsignetsim.model.math.MathException import MathException
from libsignetsim.settings.Settings import Settings
from libsignetsim.model.math.sympy_shortcuts import (
	SympySymbol, SympyEqual, SympyUnequal, SympyGreaterThan, SympyLessThan,
	SympyStrictGreaterThan, SympyStrictLessThan, SympyAnd, SympyOr, SympyXor, SympyTrue, SympyFalse
)

from libsignetsim.model.math.MathDevelopper import unevaluatedSubs

from libsbml import parseL3Formula
from sympy import srepr, simplify


class EventTrigger(MathFormula, SbmlObject):
	""" Events definition """

	def __init__(self, model, math_only=False):

		self.__model = model

		MathFormula.__init__(self, model, MathFormula.MATH_EQUATION)
		self.initialValue = False
		self.isPersistent = True
		self.mathOnly = math_only
		if not self.mathOnly:
			SbmlObject.__init__(self, model)

	def readSbml(self, sbml_trigger, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Reads an event definition from a sbml file """

		SbmlObject.readSbml(self, sbml_trigger, sbml_level, sbml_version)
		MathFormula.readSbml(self, sbml_trigger.getMath(), sbml_level, sbml_version)
		MathFormula.setInternalMathFormula(self, MathFormula.ensureBool(self, MathFormula.getInternalMathFormula(self)))
		if sbml_level >= 3:
			self.isPersistent = sbml_trigger.getPersistent()
			self.initialValue = sbml_trigger.getInitialValue()

	def writeSbml(self, sbml_event, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Writes an event definition to a sbml file """

		sbml_trigger = sbml_event.createTrigger()
		SbmlObject.writeSbml(self, sbml_trigger, sbml_level, sbml_version)

		sbml_trigger.setMath(MathFormula.writeSbml(self, sbml_level, sbml_version))

		if sbml_level >= 3:
			sbml_trigger.setPersistent(self.isPersistent)
			sbml_trigger.setInitialValue(self.initialValue)

		sbml_event.setTrigger(sbml_trigger)

	def copy(self, obj, symbols_subs={}, conversion_factors={}):

		if not self.mathOnly:
			SbmlObject.copy(self, obj)

		t_convs = {}
		for var, conversion in list(conversion_factors.items()):
			t_convs.update({var: var/conversion})

		t_formula = unevaluatedSubs(obj.getInternalMathFormula(rawFormula=False), symbols_subs)
		t_formula = unevaluatedSubs(t_formula, t_convs)
		MathFormula.setInternalMathFormula(self, t_formula)

		self.initialValue = obj.initialValue
		self.isPersistent = obj.isPersistent

	def copySubmodel(self, obj):

		MathFormula.setInternalMathFormula(self, obj.getDeveloppedInternalMathFormula())
		self.initialValue = obj.initialValue
		self.isPersistent = obj.isPersistent

	def setPrettyPrintMathFormula(self, string, rawFormula=False):
		""" Sets the formula for substance. If forcedConcentration, the species are replaced by their concentration"""

		sbml_formula = parseL3Formula(str(string))
		if sbml_formula is None:
			raise SbmlException("MathFormula : Unable to parse math formula")

		MathFormula.readSbml(self, sbml_formula, self.sbmlLevel, self.sbmlVersion)
		if not rawFormula:
			t_subs_mask = {}
			for t_var in self.__model.listOfSpecies:
				if t_var.isConcentration():
					t_symbol = SympySymbol("_speciesForcedConcentration_%s_" % str(t_var.symbol.getInternalMathFormula()))
					t_subs_mask.update({t_var.symbol.getInternalMathFormula(): t_symbol})

			MathFormula.setInternalMathFormula(self, MathFormula.getInternalMathFormula(self).subs(t_subs_mask))

	def getRootsFunctions(self):
		return self.generateRootsFunctions(simplify(MathFormula.getDeveloppedInternalMathFormula(self)))

	def generateRootsFunctions(self, tree):

		if tree.func in [SympyAnd, SympyOr, SympyXor]:
			return (
				self.generateRootsFunctions(tree.args[0])
				+ self.generateRootsFunctions(tree.args[1])
			)

		elif tree == SympyTrue:
			return ["(1)"]

		elif tree == SympyFalse:
			return ["(0)"]

		else:
			if tree.func in [SympyLessThan, SympyStrictLessThan]:
				return ["(%s - %s)" % (
					MathFormula.writeCCode(self, tree.args[1]),
					MathFormula.writeCCode(self, tree.args[0])
				)]
			else:

				return ["(%s - %s)" % (
					MathFormula.writeCCode(self, tree.args[0]),
					MathFormula.writeCCode(self, tree.args[1])
				)]

	def getDeactivationCondition(self, shift=0):

			i_event = shift
			(res, i_event) = self.generateDeactivationCondition(MathFormula.getDeveloppedInternalMathFormula(self), i_event)
			return res, i_event

	def generateDeactivationCondition(self, tree, i_event):

		res_i_event = i_event

		if tree.func == SympyAnd:
			(t_cond_0, res_i_event) = self.generateDeactivationCondition(tree.args[0], res_i_event)
			(t_cond_1, res_i_event) = self.generateDeactivationCondition(tree.args[1], res_i_event)
			return ("(%s || %s)" % (t_cond_0, t_cond_1) , res_i_event)

		elif tree.func == SympyOr:
			(t_cond_0, res_i_event) = self.generateDeactivationCondition(tree.args[0], res_i_event)
			(t_cond_1, res_i_event) = self.generateDeactivationCondition(tree.args[1], res_i_event)
			return ("(%s && %s)" % (t_cond_0, t_cond_1) , res_i_event)

		elif tree.func == SympyXor:
			(t_cond_0, res_i_event) = self.generateDeactivationCondition(tree.args[0], res_i_event)
			(t_cond_1, res_i_event) = self.generateDeactivationCondition(tree.args[1], res_i_event)
			return ("(%s && %s) || (!%s && !%s))" % (t_cond_0, t_cond_1, t_cond_0, t_cond_1) , res_i_event)

		else:
			return ("(data->roots_triggers[%d] == -1)" % res_i_event, res_i_event+1)

	def getActivationCondition(self, shift=0):

			i_event = shift
			(res, i_event) = self.generateActivationCondition(
								simplify(MathFormula.getDeveloppedInternalMathFormula(self)),
								i_event)
			return res, i_event

	def generateActivationCondition(self, tree, i_event):

		res_i_event = i_event

		if tree.func == SympyAnd:
			(t_cond_0, res_i_event) = self.generateActivationCondition(tree.args[0], res_i_event)
			(t_cond_1, res_i_event) = self.generateActivationCondition(tree.args[1], res_i_event)
			return ("(%s && %s)" % (t_cond_0, t_cond_1) , res_i_event)

		elif tree.func == SympyOr:
			(t_cond_0, res_i_event) = self.generateActivationCondition(tree.args[0], res_i_event)
			(t_cond_1, res_i_event) = self.generateActivationCondition(tree.args[1], res_i_event)
			return ("(%s || %s)" % (t_cond_0, t_cond_1) , res_i_event)

		elif tree.func == SympyXor:
			(t_cond_0, res_i_event) = self.generateActivationCondition(tree.args[0], res_i_event)
			(t_cond_1, res_i_event) = self.generateActivationCondition(tree.args[1], res_i_event)
			return ("(%s || %s) && (!(%s && %s))" % (t_cond_0, t_cond_1, t_cond_0, t_cond_1) , res_i_event)

		else:
			return ("(data->roots_triggers[%d] == 1)" % res_i_event, res_i_event+1)

	def nbRoots(self):
		return self.countRoots(simplify(MathFormula.getDeveloppedInternalMathFormula(self)), 0)

	def countRoots(self, tree, counter):

		res_counter = counter

		if tree.func in [SympyAnd, SympyOr, SympyXor]:
			res_counter = self.countRoots(tree.args[0], res_counter)
			res_counter = self.countRoots(tree.args[1], res_counter)
			return res_counter

		else:
			return res_counter + 1

	def getRootsOperator(self):
		return self.generateRootsOperator(simplify(MathFormula.getDeveloppedInternalMathFormula(self)), [])

	def generateRootsOperator(self, tree, t_list):

		if tree.func in [SympyAnd, SympyOr, SympyXor]:
			t_list += self.generateRootsOperator(tree.args[0], t_list)
			t_list += self.generateRootsOperator(tree.args[1], t_list)
			return t_list

		else:
			if tree.func in [SympyStrictGreaterThan, SympyStrictLessThan]:
				return [1]
			elif tree.func in [SympyGreaterThan, SympyLessThan]:
				return [0]
			elif tree.func == SympyEqual:
				return [2]
			elif tree.func == SympyUnequal:
				return [3]
			elif tree == SympyTrue or tree == SympyFalse:
				return [3]
			else:
				raise MathException("Event Trigger : Unknown logical operator %s" % srepr(tree.func))

	def getOperator(self):
		return MathFormula.getInternalMathFormula(self).func

	def isValid(self):
		return MathFormula.getInternalMathFormula(self) is not None


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function

from libsignetsim.model.math.CFE import CFE
from libsignetsim.model.math.ODE import ODE
from libsignetsim.model.math.MathSubmodel import MathSubmodel
from libsignetsim.model.math.MathVariable import MathVariable
from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.math.sympy_shortcuts import SympySymbol, SympyInteger, SympyEqual
from libsignetsim.model.math.MathStoichiometryMatrix import MathStoichiometryMatrix
from libsignetsim.model.math.container.ListOfConservationLaws import ListOfConservationLaws
from libsignetsim.model.math.MathDevelopper import unevaluatedSubs

from sympy import solve, pprint, pretty


class MathSlowModel(MathSubmodel):
	""" Sbml model class """

	def __init__(self, sbml_model=None, reduced_model=None):
		""" Constructor of model class """

		MathSubmodel.__init__(self, parent_model=reduced_model)

		self.sbmlModel = sbml_model
		self.reducedModel = reduced_model

		self.fastLaws = []
		self.fastLaws_vars = []

		self.fastStoichiometryMatrix = MathStoichiometryMatrix(sbml_model)
		self.slowStoichiometryMatrix = MathStoichiometryMatrix(sbml_model)
		self.fastConservationLaws = ListOfConservationLaws(sbml_model)
		self.DEBUG = False

	def copyEquations(self):

		for cfe in self.reducedModel.listOfCFEs:
			t_var = self.listOfVariables.getBySymbol(cfe.getVariable().symbol.getSymbol())
			t_cfe_formula = MathFormula(self)
			t_cfe_formula.setInternalMathFormula(cfe.getDefinition().getDeveloppedInternalMathFormula())
			t_cfe = CFE(self)
			t_cfe.new(t_var, t_cfe_formula)
			self.listOfCFEs.append(t_cfe)

		for ode in self.reducedModel.listOfODEs:
			original_variable = self.sbmlModel.listOfVariables.getBySymbol(ode.getVariable().symbol.getSymbol())
			if original_variable.isRateRuled():
				t_var = self.listOfVariables.getBySymbol(ode.getVariable().symbol.getSymbol())
				t_ode_formula = MathFormula(self)
				t_ode_formula.setInternalMathFormula(ode.getDefinition().getDeveloppedInternalMathFormula())
				t_ode = ODE(self)
				t_ode.new(t_var, t_ode_formula)
				self.listOfODEs.append(t_ode)

	def findFastReactions(self):
		""" Finds the fast reactions and build the fast stoichiometry matrix """

		for reaction in self.sbmlModel.listOfReactions:
			if reaction.fast:
				self.fastLaws.append(reaction.kineticLaw.getDefinition().getDeveloppedInternalMathFormula())

				for reactant in reaction.listOfReactants:
					self.fastLaws_vars.append(reactant.getSpecies().symbol.getDeveloppedInternalMathFormula())

				for product in reaction.listOfProducts:
					self.fastLaws_vars.append(product.getSpecies().symbol.getDeveloppedInternalMathFormula())

	def findVelocities(self, subs={}, include_fast_reaction=True, include_slow_reaction=True):

		matrix_velocities = None
		for reaction in self.sbmlModel.listOfReactions:
			velocities = reaction.kineticLaw.getRawVelocities(
				subs=subs,
				include_fast_reaction=include_fast_reaction,
				include_slow_reaction=include_slow_reaction
			)

			if matrix_velocities is None:
				matrix_velocities = velocities
			else:
				matrix_velocities = matrix_velocities.col_join(velocities)

		return matrix_velocities

	def __findKeptVariables(self):

		original_variables = [var.symbol.getSymbol() for var in self.reducedModel.variablesOdes]
		kept_variables = []
		for i, var in enumerate(self.sbmlModel.variablesOdes):
			if var.symbol.getSymbol() in original_variables:
				kept_variables.append(i)

		return kept_variables

	def __buildStoichiometryMatrices(self, kept_variables):

		self.fastStoichiometryMatrix.build(including_slow_reactions=False)
		self.slowStoichiometryMatrix.build(including_fast_reactions=False)
		self.findFastReactions()

		fast_matrix = self.fastStoichiometryMatrix.stoichiometryMatrix
		slow_matrix = self.slowStoichiometryMatrix.stoichiometryMatrix

		return (fast_matrix[kept_variables, :], slow_matrix[kept_variables, :])

	def __buildODEs(self, slow_system, slow_variables, subs):

		for i, symbol in enumerate(slow_variables):
			variable = self.listOfVariables.getBySymbol(symbol)

			t_definition = MathFormula(self)
			t_definition.setInternalMathFormula(unevaluatedSubs(sum(slow_system[i, :]), subs))

			ode = ODE(self)
			ode.new(variable, t_definition)
			self.listOfODEs.append(ode)

	def __buildCFEs(self, fast_laws, fast_vars, subs):

		for law in fast_laws:

			law = unevaluatedSubs(law, subs)
			intersect = set(fast_vars).intersection(set(law.atoms(SympySymbol)))
			if len(intersect) > 0:
				var = list(intersect)[0]

				if self.DEBUG:
					print(pretty(law))
					print(pretty(var))

				res = solve(law, var)

				t_var = self.listOfVariables.getBySymbol(var)
				t_cfe = MathFormula(self)
				t_cfe.setInternalMathFormula(res[0])

				cfe = CFE(self)
				cfe.new(t_var, t_cfe)

				self.listOfCFEs.append(cfe)
				self.listOfVariables.changeVariableType(t_var, MathVariable.VAR_ASS)

		self.listOfCFEs.developCFEs()

	def __classifyVariables(self, kept_variables):

		slow_variables = []
		fast_variables = []
		for var in kept_variables:
			variable = self.sbmlModel.variablesOdes[var]
			if variable.isSpecies() and not variable.isOnlyInFastReactions():
				slow_variables.append(variable.symbol.getSymbol())
			else:
				fast_variables.append(variable.symbol.getSymbol())

		return slow_variables, fast_variables

	def build(self):

		if self.DEBUG:
			print(">> Building slow model")

		self.copyVariables()
		self.copyEquations()

		kept_variables = self.__findKeptVariables()
		if self.DEBUG:
			print("> Variables kept in the reduced model : %s" % str(kept_variables))

		fast_matrix, slow_matrix = self.__buildStoichiometryMatrices(kept_variables)

		fast_velocities = self.findVelocities(include_slow_reaction=False)
		slow_velocities = self.findVelocities(include_fast_reaction=False)

		fast_system = fast_matrix * fast_velocities
		slow_system = slow_matrix * slow_velocities

		if self.DEBUG:
			print("> Fast system matrix")
			pprint(fast_system)
			print("\n")

			print("> Slow system matrix")
			pprint(slow_system)
			print("\n")

		subs = self.reducedModel.listOfCFEs.getSubs()
		slow_variables, fast_variables = self.__classifyVariables(kept_variables)

		self.__buildODEs(slow_system, slow_variables, subs)
		self.__buildCFEs(self.fastLaws, fast_variables, subs)

		# Computing new initial values
		system_vars = self.fastLaws_vars

		subs = {}
		for var, math_formula in list(self.sbmlModel.listOfInitialConditions.items()):
			variable = self.sbmlModel.listOfVariables.getBySymbol(var)
			if var not in system_vars or variable.boundaryCondition:
				subs.update({var: math_formula.getDeveloppedInternalMathFormula()})

		if self.DEBUG:
			print("> Known initial values : %s" % pretty(subs))

		system = []

		# Solving the initial values using the fast conservation laws, and the fast laws.
		for cons_law in self.fastConservationLaws.getRawFormulas(self.fastStoichiometryMatrix):
			formula = unevaluatedSubs(cons_law, subs)
			if formula not in [True, False]:
				system.append(formula)

		for fast_law in self.fastLaws:
			formula = SympyEqual(unevaluatedSubs(fast_law, subs), SympyInteger(0))
			if formula not in [True, False]:
				system.append(formula)

		if self.DEBUG:
			print(system)
			print(system_vars)

		res = solve(system, system_vars)

		if self.DEBUG:
			print(res)

		for var, value in list(res.items()):
			math_formula = MathFormula(self)
			math_formula.setInternalMathFormula(unevaluatedSubs(value, subs))

			self.listOfInitialConditions.update({var: math_formula})

		self.setUpToDate(True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.settings.Settings import Settings
from libsignetsim.sedml.SedmlException import SedmlUnknownURI
from libsignetsim.sedml.math.sympy_shortcuts import SympySymbol

class URI(object):

	SEDML = "sedml"
	MIRIAM = "miriam"

	BIOMODELS = "biomodels.db"
	SYMBOL = "symbol"

	def __init__(self, document):
		self.__document = document

		self.__type = None
		self.__tokens = None

		self.__symbol = None

		self.__url = None
		self.__filename = None

	def readSedml(self, uri, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		self.parseURI(uri)


	def writeSedml(self, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		if self.__tokens is not None:
			return ":".join(self.__tokens)


	def parseURI(self, uri):
		if uri.startswith("urn:"):

			self.__tokens = uri.split(":")

			if len(self.__tokens) == 4 and self.__tokens[1] == self.MIRIAM and self.__tokens[2] == self.BIOMODELS:
				self.__url = "http://www.ebi.ac.uk/biomodels-main/download?mid=%s" % self.__tokens[3]
				self.__filename = "%s.xml" % self.__tokens[3]
				self.__type = self.BIOMODELS

			elif len(self.__tokens) == 4 and self.__tokens[1] == self.SEDML and self.__tokens[2] == self.SYMBOL:
				if self.__tokens[3] == 'time':
					self.__type = self.SYMBOL
					self.__symbol = SympySymbol("time")

				else:
					raise SedmlUnknownURI("Unknown symbol %s" % self.__tokens[3])

			else:
				raise SedmlUnknownURI("Unknown URI %s" % uri)


		else:
			raise SedmlUnknownURI(uri)

	def getURL(self):
		return self.__url

	def getFilename(self):
		return self.__filename

	def setURI(self, uri):
		self.parseURI(uri)

	def getSymbol(self):
		if self.__type == self.SYMBOL and self.__symbol is not None:
			return self.__symbol

	def isTime(self):
		return self.__type == self.SYMBOL and self.__symbol is not None and self.__symbol == SympySymbol("time")

	def setTime(self):
		self.parseURI("urn:sedml:symbol:time")


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Testing the creation and destruction of local parameters

"""

from libsignetsim import SbmlDocument, Model, KineticLaw, Settings

from unittest import TestCase
from os.path import join

class TestLocalParameters(TestCase):
	""" Tests local parameters """


	def testLocalParameter(self):
		""" Just create and local parameter, and then destroy it
		"""

		model = Model()

		c = model.listOfCompartments.new("c", value=2)

		s1 = model.listOfSpecies.new("s1")

		p1 = model.listOfParameters.new("p1")

		r1 = model.listOfReactions.new("reaction 1")
		r1.listOfReactants.add(s1)
		r1.setKineticLaw(KineticLaw.MASS_ACTION, reversible=False, parameters=[p1])

		self.assertEqual(len(r1.listOfLocalParameters), 0)

		lp1 = r1.listOfLocalParameters.new("lp1")

		self.assertEqual(len(r1.listOfLocalParameters), 1)
		self.assertEqual(r1.listOfLocalParameters[0].getName(), "lp1")

		model.parentDoc.writeSbmlToFile(join(Settings.tempDirectory, "model.sbml"))
		new_doc = SbmlDocument()
		new_doc.readSbmlFromFile(join(Settings.tempDirectory, "model.sbml"))
		model_2 = new_doc.getModelInstance()

		self.assertEqual(len(model_2.listOfReactions[0].listOfLocalParameters), 1)


		r1.listOfLocalParameters.remove(lp1)
		self.assertEqual(len(r1.listOfLocalParameters), 0)

		model.parentDoc.writeSbmlToFile(join(Settings.tempDirectory, "model.sbml"))
		new_doc = SbmlDocument()
		new_doc.readSbmlFromFile(join(Settings.tempDirectory, "model.sbml"))
		model_2 = new_doc.getModelInstance()

		self.assertEqual(len(model_2.listOfReactions[0].listOfLocalParameters), 0)
		self.assertEqual(len(model.listOfParameters), 1)

		p1.toLocal(r1)
		self.assertEqual(len(model.listOfReactions[0].listOfLocalParameters), 1)
		self.assertEqual(len(model.listOfParameters), 0)

		p1.toGlobal()
		self.assertEqual(len(model.listOfReactions[0].listOfLocalParameters), 0)
		self.assertEqual(len(model.listOfParameters), 1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.numl.Dimension import Dimension
from libsignetsim.numl.TupleValue import TupleValue
from libsignetsim.numl.AtomicValue import AtomicValue
from libsignetsim.settings.Settings import Settings

class CompositeValue (Dimension):

	def __init__(self, document, result_component, description=None, index_value=""):

		Dimension.__init__(self, document, result_component, description)
		self.__document = document
		self.__contents = []
		self.__indexValue = index_value

	def createCompositeValue(self, description=None, index_value=""):

		from libsignetsim.numl.CompositeValue import CompositeValue
		composite_value = CompositeValue(self.__document, self.getResultComponent(), description, index_value)
		self.__contents.append(composite_value)
		return composite_value

	def createTupleValue(self, description=None):

		tuple_value = TupleValue(self.__document, self.getResultComponent(), description)
		self.__contents.append(tuple_value)
		return tuple_value


	def createAtomicValue(self, description=None, value=None):

		atomic_value = AtomicValue(self.__document, self.getResultComponent(), description, value)
		self.__contents.append(atomic_value)
		return atomic_value

	def readNuML(self, composite_value, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):
		Dimension.readNuML(self, composite_value)

		for composite_value_element in composite_value:
			if composite_value.isContentCompositeValue():
				from libsignetsim.numl.CompositeValue import CompositeValue
				t_content = CompositeValue(self.__document, self.getResultComponent(), self.getDescription().getContent())
				t_content.readNuML(composite_value_element, level, version)
				self.__contents.append(t_content)

			elif composite_value.isContentTuple():
				t_content = TupleValue(self.__document, self.getResultComponent(), self.getDescription().getContent())
				t_content.readNuML(composite_value_element, level, version)
				self.__contents.append(t_content)

			elif composite_value.isContentAtomicValue():
				t_content = AtomicValue(self.__document, self.getResultComponent(), self.getDescription().getContent())
				t_content.readNuML(composite_value_element, level, version)
				self.__contents.append(t_content)

		self.__indexValue = composite_value.getIndexValue()

	def writeNuML(self, dimension, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):
		Dimension.writeNuML(self, dimension)
		if self.getDescription() is not None:
			dimension.setDescription(self.getDescription().getId())

		for content in self.__contents:
			t_dimension = None

			from libsignetsim.numl.CompositeValue import CompositeValue
			if isinstance(content, CompositeValue):
				t_dimension = dimension.createCompositeValue()

			elif isinstance(content, TupleValue):
				t_dimension = dimension.createTuple()

			elif isinstance(content, AtomicValue):
				t_dimension = dimension.createAtomicValue()

			if t_dimension is not None:
				content.writeNuML(t_dimension, level, version)

		dimension.setIndexValue(str(self.__indexValue))

	def getContents(self):
		return self.__contents

	def getIndexValue(self):
		return self.__indexValue


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.container.ListOf import ListOf
from libsignetsim.model.sbml.SbmlObject import SbmlObject

from libsignetsim.model.sbml.Rule import Rule
from libsignetsim.model.sbml.AssignmentRule import AssignmentRule
from libsignetsim.model.sbml.RateRule import RateRule
from libsignetsim.model.sbml.AlgebraicRule import AlgebraicRule
from libsignetsim.settings.Settings import Settings
from libsbml import RULE_TYPE_SCALAR, RULE_TYPE_RATE

class ListOfRules(ListOf, SbmlObject):
	""" Container for the rules of a sbml model"""

	def __init__ (self, model=None):

		self.__model = model
		ListOf.__init__(self, model)
		SbmlObject.__init__(self, model)

	def readSbml(self, sbml_listOfRules,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		""" Reads rules' list from a sbml file """

		for sbml_rule in sbml_listOfRules:
			if sbml_level >= 2:
				if sbml_rule.isAssignment():
					t_rule = AssignmentRule(self.__model, self.nextId())
				elif sbml_rule.isRate():
					t_rule = RateRule(self.__model, self.nextId())
				elif sbml_rule.isAlgebraic():
					t_rule = AlgebraicRule(self.__model, self.nextId())
				else:
					t_rule = Rule(self.__model, self.nextId())
			else:
				if sbml_rule.getType() == RULE_TYPE_SCALAR:
					t_rule = AssignmentRule(self.__model, self.nextId())
				elif sbml_rule.getType() == RULE_TYPE_RATE:
					t_rule = RateRule(self.__model, self.nextId())
				else:
					t_rule = AlgebraicRule(self.__model, self.nextId())

			t_rule.readSbml(sbml_rule, sbml_level, sbml_version)
			ListOf.add(self, t_rule)

		SbmlObject.readSbml(self, sbml_listOfRules, sbml_level, sbml_version)

	def writeSbml(self, sbml_model,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		""" Writes rules' list to a sbml file """

		for rule in self:
			rule.writeSbml(sbml_model, sbml_level, sbml_version)

		if len(self):
			SbmlObject.writeSbml(self, sbml_model.getListOfRules(), sbml_level, sbml_version)

	def copy(self, obj, deletions=[], sids_subs={}, symbols_subs={}, conversion_factors={}, time_conversion=None):

		if obj not in deletions:
			SbmlObject.copy(self, obj)

			for sbml_rule in obj:

				if sbml_rule not in deletions:

					if sbml_rule.isAssignment():
						t_rule = AssignmentRule(self.__model, self.nextId())
						t_rule.copy(sbml_rule, sids_subs=sids_subs, symbols_subs=symbols_subs, conversion_factors=conversion_factors)
					elif sbml_rule.isRate():
						t_rule = RateRule(self.__model, self.nextId())
						t_rule.copy(sbml_rule, sids_subs=sids_subs, symbols_subs=symbols_subs, conversion_factors=conversion_factors, time_conversion=time_conversion)
					elif sbml_rule.isAlgebraic():
						t_rule = AlgebraicRule(self.__model, self.nextId())
						t_rule.copy(sbml_rule, symbols_subs=symbols_subs)

					ListOf.add(self, t_rule)

	def getByVariable(self, variable, pos=0):
		""" Finds rules by variable """
		return [r for r in self if r.variable.getSbmlMathFormula() == variable][pos]

	def hasAssignmentRule(self):
		""" Checks if there is at least one algebraic rule """
		for rule in self:
			if rule.isAssignment():
				return True
		return False

	def hasAlgebraicRule(self):
		""" Checks if there is at least one algebraic rule """
		for rule in self:
			if rule.isAlgebraic():
				return True

		if self.__model.listOfReactions.hasFastReaction():
			return True
		return False

	def countAlgebraicRules(self):
		nb = 0
		for rule in self:
			if rule.isAlgebraic():
				nb += 1

		return nb

	def algebraicContainsVariable(self, variable):
		for rule in self:
			if rule.isAlgebraic():
				if rule.containsVariable(variable):
					return True
		return False

	def newAlgebraicRule(self, expression=None, rawFormula=False):
		return self.new(Rule.RULE_ALGEBRAIC, expression=expression, rawFormula=rawFormula)

	def newAssignmentRule(self, variable=None, expression=None, rawFormula=False):
		return self.new(Rule.RULE_ASSIGNMENT, variable=variable, expression=expression, rawFormula=rawFormula)

	def newRateRule(self, variable=None, expression=None, rawFormula=False):
		return self.new(Rule.RULE_RATE, variable=variable, expression=expression, rawFormula=rawFormula)

	def new(self, rule_type, variable=None, expression=None, rawFormula=False):

		t_rule = None

		if rule_type == Rule.RULE_ALGEBRAIC:
			t_rule = AlgebraicRule(self.__model, self.nextId())
			if expression is not None:
				t_rule.setPrettyPrintDefinition(expression, rawFormula=rawFormula)

		elif rule_type == Rule.RULE_ASSIGNMENT:
			t_rule = AssignmentRule(self.__model, self.nextId())
			if (variable is not None and expression is not None):
				t_rule.setVariable(variable)
				t_rule.setPrettyPrintDefinition(expression, rawFormula=rawFormula)

		elif rule_type == Rule.RULE_RATE:
			t_rule = RateRule(self.__model, self.nextId())
			if (variable is not None and expression is not None):
				t_rule.setVariable(variable)
				t_rule.setPrettyPrintDefinition(expression, rawFormula=rawFormula)

		ListOf.add(self, t_rule)
		SbmlObject.new(t_rule)
		return t_rule

	def renameSbmlId(self, old_sbml_id, new_sbml_id):
		for rule in self:
			rule.renameSbmlId(old_sbml_id, new_sbml_id)

	def containsVariable(self, variable):

		for rule in self:
			if rule.containsVariable(variable):
				return True
		return False

	def remove(self, rule):
		""" Remove a rule from the list """

		if rule.isAssignment() or rule.isRate():
			rule.getVariable().unsetRuledBy()

		ListOf.remove(self, rule)

	def removeById(self, rule_obj_id):
		""" Remove a rule from the list """

		self.remove(self.getById(rule_obj_id))
